from PySide6.QtWidgets import QMainWindow, QTableWidgetItem, QMenu
from PySide6.QtCore import Qt

from debyetools.tpropsgui.ui_heatcapacitywindow import Ui_MainWindow as Ui_Cp

from debyetools.fs_compound_db import fit_FS as dt_fit_FS
#from debyetools.fs_compound_db import Cp2fit as dt_Cp2fit

# from  debyetools.tpropsgui.backend_qt_patched.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
# from matplotlib.backends.backend_qt5agg import FigureCanvasAgg as FigureCanvas
# from matplotlib.backends.backend_mixed import FigureCanvas
# print(hola)

# from matplotlib.backends.backend_qtagg import FigureCanvas
# from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
from matplotlib.widgets import Cursor
from matplotlib.figure import Figure

import numpy as np

class windowCp(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_Cp()
        self.ui.setupUi(self)
        # self.app = app

        self.fig = Figure(figsize=(3, 3))
        self.canvas = FigureCanvas(self.fig)
        self.set_canvas_configuration()

        llayout = self.ui.horizontalLayout#QHBoxLayout()
        llayout.addWidget(self.canvas, 88)
        self.canvas.setContextMenuPolicy(Qt.CustomContextMenu)
        self.canvas.customContextMenuRequested.connect(self.right_menu)

        self.ui.pushButton.clicked.connect(self.on_click_recalc)
        self.ui.lineEdit.textChanged.connect(self.update_FS_Tto)
        self.ui.comboBox.currentIndexChanged.connect(self.selectionchange)

        self.ui.comboBox_2.currentIndexChanged.connect(self.selectionchange_plot)
        self.current_plot = 9

        self.ui.progress.setGeometry(700, 505, 81, 5)



    def selectionchange_plot(self, i):
        self.current_plot = i
        self.plot_prop('T', self.proplist[i])



    def selectionchange(self, i):
        Pi = self.Ps[i]
        self.ui.tableWidget.setItem(0,0,QTableWidgetItem('%.5e' % (self.dict_H298['%.1f'%(Pi/1e9)])))
        self.ui.tableWidget.setItem(1,0,QTableWidgetItem('%.5e' % (self.dict_S298['%.1f'%(Pi/1e9)])))
        for ix, p in enumerate(self.dict_FS['%.1f'%(Pi/1e9)]['Cp']):
            self.ui.tableWidget.setItem(ix+2,0,QTableWidgetItem('%.5e' % (p)))



    def get_T(self):
        T_initial, T_final, Tstep = (float(sti) for sti in self.ui.lineEdit.text().split())
        T = np.arange(T_initial, T_final+Tstep, Tstep)
        T = np.r_[T, [298.15]]
        T.sort()

        return T

    def get_P(self):
        Ps = self.ui.lineEdit_2.text().split()
        if len(Ps)==1:
            return np.array([float(sti) for sti in Ps])
        T_initial, T_final, Tstep = (float(sti) for sti in self.ui.lineEdit_2.text().split())
        T = np.arange(T_initial, T_final+Tstep, Tstep)
        T.sort()

        return T

    def get_FS_T(self):
        return float(self.ui.lineEdit_3.text()), float(self.ui.lineEdit_4.text())

    def set_canvas_configuration(self):
        self.canvas.figure.set_constrained_layout(True)
        self.fig.set_canvas(self.canvas)
        self._ax = self.canvas.figure.add_subplot(111)
        self.cursor = Cursor(self._ax, useblit=True, linewidth=0.8, color='lightgray')


    def plot_prop(self, str_x, str_y):
        import matplotlib.cm as cm
        self._ax.cla()

        self.data_dict={pi_str:{} for pi_str in self.dict_tp.keys()}

        len_Ps = len(self.dict_tp.keys())
        for i, Pi_str in enumerate(self.dict_tp.keys()):
            tprops_dict = self.dict_tp[Pi_str]

            X = tprops_dict[str_x]
            Y = tprops_dict[str_y]

            self.data_dict[Pi_str] = {str_x: X, str_y:Y}

            c = cm.PuRd((i+1)/len_Ps,1)
            self._ax.plot(X, Y, label='debyetools', color=c)
            self._ax.text(X[-1]-100,Y[-1],'P='+Pi_str+'GPa', size=5)

        self._ax.set_xlabel(str_x)
        self._ax.set_ylabel(str_y)

        self.canvas.draw()


    def debye_run(self, molecule, ui_progress):
        if self.ui.radioButton.isChecked():
            mode ='jjsl'
        elif self.ui.radioButton_2.isChecked():
            mode = 'jjdm'
        elif self.ui.radioButton_3.isChecked():
            mode = 'jjfv'
        self.ui.comboBox.clear()
        self.molecule = molecule
        molecule.initialize_ndeb(mode)
        self.ndeb_obj = molecule.ndeb

        T = self.get_T()
        Ps = self.get_P()*1e9
        self.Ps = Ps


        self.dict_tp = {'%.1f'%(Pi/1e9):'' for Pi in Ps}
        self.dict_FS = {'%.1f'%(Pi/1e9):{'Cp':[0]} for Pi in Ps}
        self.dict_H298 = {'%.1f'%(Pi/1e9):0 for Pi in Ps}
        self.dict_S298 = {'%.1f'%(Pi/1e9):-1 for Pi in Ps}



        lP = len(Ps)
        progress = (0)/lP
        ui_progress.setValue(progress*100)
        for ix, P in enumerate(Ps):

            self.ui.comboBox.addItem('%.1f'%(P/1e9))

            molecule.min_G(T, P)

            molecule.eval_props()

            progress =(ix+1)/lP
            ui_progress.setValue(progress*100)

            self.dict_tp['%.1f'%(P/1e9)] =  molecule.tprops_dict

            self.FS_Tfrom, self.FS_Tto = self.get_FS_T()

            self.dict_FS['%.1f'%(P/1e9)] = dt_fit_FS(molecule.tprops_dict, self.FS_Tfrom, self.FS_Tto)

            ix_T0 = np.where(np.round(molecule.tprops_dict['T'],2) == np.round(298.15,2))[0][0]

            self.dict_H298['%.1f'%(P/1e9)] = molecule.tprops_dict['G'][ix_T0]+molecule.tprops_dict['T'][ix_T0]*molecule.tprops_dict['S'][ix_T0]
            self.dict_S298['%.1f'%(P/1e9)] = molecule.tprops_dict['S'][ix_T0]

#            self.plot_Cp(self.dict_tp['%.1f'%(P/1e9)], self.dict_FS['%.1f'%(P/1e9)])

        self.ui.tableWidget.setItem(0,0,QTableWidgetItem('%.5e' % (self.dict_H298['%.1f'%(0/1e9)])))
        self.ui.tableWidget.setItem(1,0,QTableWidgetItem('%.5e' % (self.dict_S298['%.1f'%(0/1e9)])))
        for ix, p in enumerate(self.dict_FS['%.1f'%(0/1e9)]['Cp']):
            self.ui.tableWidget.setItem(ix+2,0,QTableWidgetItem('%.5e' % (p)))

#            self._ax.text(Y[-1],X[-1],'P=')


#        print('xxxx', list(self.dict_tp[list(self.dict_tp.keys())[0]].keys()))
        self.proplist = list(self.dict_tp[list(self.dict_tp.keys())[0]].keys())
        current_plot = self.current_plot
        self.ui.comboBox_2.clear()

        for k in self.proplist:
            self.ui.comboBox_2.addItem(k)
        self.current_plot = current_plot

        self.ui.comboBox_2.setCurrentIndex(self.current_plot)

        self.plot_prop('T', self.proplist[self.current_plot])


#        handles_1, labels_1 = self._ax.get_legend_handles_labels()
#        by_label_1 = dict(zip(labels_1, handles_1))
#        self._ax.legend(by_label_1.values(), by_label_1.keys())

    def right_menu(self, pos):
         menu = QMenu()

         # Add menu options
         hello_option = menu.addAction('Copy data')

         # Menu option events
         hello_option.triggered.connect(self.on_click_copydata)

         # Position
         menu.exec(self.mapToGlobal(pos))

    def on_click_copydata(self):
#        app = QApplication(sys.argv)

        txt2copy = ''
        for k in self.data_dict:
            txt2copy = txt2copy + '#P = '+ k +' GPa\n'
            txt2copy = txt2copy + '#'
            for ki in self.data_dict[k].keys():
                txt2copy = txt2copy + ki + '    '
            txt2copy = txt2copy + '\n'
            data=[]
            for ki in self.data_dict[k].keys():
                data.append(self.data_dict[k][ki])

            data=np.array(data).T
            for d in data:
                for di in d:
                    txt2copy = txt2copy + str(di) + '    '
                txt2copy = txt2copy + '\n'
        clipboard=self.app.clipboard()
        clipboard.setText(txt2copy)



    def on_click_recalc(self):
        self._ax.cla()
        self.debye_run(self.molecule, self.ui.progress)

    def update_FS_Tto(self):
        try:
            Tf =self.get_T()[-1]
            self.ui.lineEdit_4.setText(str(Tf))
        except:
            pass



import debyetools.tpropsgui.elements as elmt

lo_fs_Cp = lambda i : [[elmt.T('C_P:','fs_Cp'+str(i),pad=(3,0),right_click_menu=['',['copy to clipboard::Cp_'+str(i)]])],
                       [elmt.Ipad('','fsCp_P0'+str(i),10, pad=(0,0)),elmt.T('*T^0','fs_Cp_P0'+str(i),pad = ((0,0),(0,0)))],
                       [elmt.Ipad('','fsCp_P1'+str(i),10, pad=(0,0)),elmt.T('*T^1','fs_Cp_P1'+str(i),pad = ((0,0),(0,0)))],
                       [elmt.Ipad('','fsCp_P2'+str(i),10, pad=(0,0)),elmt.T('*T^(-2)','fs_Cp_P2'+str(i),pad = ((0,0),(0,0)))],
                       [elmt.Ipad('','fsCp_P3'+str(i),10, pad=(0,0)),elmt.T('*T^2','fs_Cp_P3'+str(i),pad = ((0,0),(0,0)))],
                       [elmt.Ipad('','fsCp_P4'+str(i),10, pad=(0,0)),elmt.T('*T^(-0.5)','fs_Cp_P4'+str(i),pad = ((0,0),(0,0)))],
                       [elmt.Ipad('','fsCp_P5'+str(i),10, pad=(0,0)),elmt.T('*T^(-3)','fs_Cp_P5'+str(i),pad = ((0,0),(0,0)))]]
lo_fs_alpha = lambda i: [[elmt.T('alpha:','fs_alpha'+str(i),pad=(3,0))],
                         [elmt.Ipad('','fsa_P0'+str(i),10, pad=(0,0)),elmt.T('*T^0','fs_a_P0'+str(i),pad = ((0,0),(0,0)))],
                         [elmt.Ipad('','fsa_P1'+str(i),10, pad=(0,0)),elmt.T('*T^1','fs_a_P1'+str(i),pad = ((0,0),(0,0)))],
                         [elmt.Ipad('','fsa_P2'+str(i),10, pad=(0,0)),elmt.T('*T^(-1)','fs_a_P2'+str(i),pad = ((0,0),(0,0)))],
                         [elmt.Ipad('','fsa_P3'+str(i),10, pad=(0,0)),elmt.T('*T^(-2)','fs_a_P3'+str(i),pad = ((0,0),(0,0)))]]
lo_fs_K = lambda i:[[elmt.T('1/K:','fs_K'+str(i),pad=(3,0))],
                    [elmt.Ipad('','fsK_P0'+str(i),10, pad=(0,0)),elmt.T('*T^0','fs_K_P0'+str(i),pad = ((0,0),(0,0)))],
                    [elmt.Ipad('','fsK_P1'+str(i),10, pad=(0,0)),elmt.T('*T^1','fs_K_P1'+str(i),pad = ((0,0),(0,0)))],
                    [elmt.Ipad('','fsK_P2'+str(i),10, pad=(0,0)),elmt.T('*T^2','fs_K_P2'+str(i),pad = ((0,0),(0,0)))],
                    [elmt.Ipad('','fsK_P3'+str(i),10, pad=(0,0)),elmt.T('*T^3','fs_K_P3'+str(i),pad = ((0,0),(0,0)))]]
lo_fs_Kp = lambda i:[[elmt.T('dK/dP','fs_Kp'+str(i),pad=(3,0))],
                     [elmt.Ipad('','fsKp_P0'+str(i),10, pad=(0,0))],
                     [elmt.Ipad('','fsKp_P1'+str(i),10, pad=(0,0))]]
def lo_fsparams(i):
    return [[elmt.T('G+TS (T=298K) = ', 'H298'+str(i), pad=((5,0),0)),elmt.Ipad('','H298'+str(i),10, pad=((0,0),0)),elmt.T('S(T=298K) = ', 'S298'+str(i), pad=((5,0),0)),elmt.Ipad('','S298'+str(i),10, pad=((0,0),0))],
            [elmt.C(lo=lo_fs_Cp(i), key='fc_Cp'+str(i)),
               elmt.C(lo=lo_fs_alpha(i), key='fc_alpha'+str(i)),
               elmt.C(lo=lo_fs_K(i), key='fc_K'+str(i)),
               elmt.C(lo=lo_fs_Kp(i), key='fc_Kp'+str(i))],
              [elmt.T('select property to plot:','fsprop2plt'+str(i)),
               elmt.ICombo(['       ','Cp','alpha','1/K','dK/dP'],'fsprop2plt'+str(i),10,1),
               elmt.Bc('Plot','plotter_fsprop2plt'+str(i),('white',elmt.theme_background_color()))]]

def layout(EOS_str_lst):
    lo_options = {EOSStr:[[elmt.T(EOSStr+' params.:','params_'+EOSStr),elmt.dI('0, 0, 0, 0','params_'+EOSStr,25),elmt.dChk('fit','calc_params_'+EOSStr,True)]] for EOSStr in EOS_str_lst}
    lo_options['MP'] = [[elmt.T('Morse cutoff:','cutoff_MP'),elmt.I('5.0','cutoff_MP',3),elmt.T('# of neigh. lvls.:','ndists_MP'),elmt.I('3','ndists_MP',3)],
                     [elmt.T('Morse params.:','params_MP'),elmt.dI('0, 0, 0','params_MP',25),elmt.dChk('fit','calc_params_MP',True)]]
    lo_options['EAM'] = [[elmt.T('EAM cutoff:','cutoff_EAM'),elmt.I('5.0','cutoff_EAM',3),elmt.T('# of neigh. lvls.:','ndists_EAM'),elmt.I('3','ndists_EAM',3)],
                     [elmt.T('EAM params.:','params_EAM'),elmt.dI('0, 0, 0','params_EAM',25),elmt.dChk('fit','calc_params_EAM',True)]]

    lo_EOS = [[elmt.listbox(['Morse','EAM','Rose-Vinet','TB-SMA','Birch-Murnaghan (3)','Mie-Gruneisen','Murnaghan (1)','Poirier-Tarantola','Birch-Murnaghan (4)','Murnaghan (2)',
                             ],'EOS_listbox')],
              [elmt.Bc('Select EOSs','add_EOS',('white', 'green'))]]

    lo_EOS_collapes = [
                [elmt.collapse(lo_options['MP'],'options_'+'MP')],
                [elmt.collapse(lo_options['EAM'],'options_'+'EAM')],
              [elmt.collapse(lo_options['RV'],'options_'+'RV')],
              [elmt.collapse(lo_options['TB'],'options_'+'TB')],
              [elmt.collapse(lo_options['BM'],'options_'+'BM')],
              [elmt.collapse(lo_options['MG'],'options_'+'MG')],
              [elmt.collapse(lo_options['MU'],'options_'+'MU')],
              [elmt.collapse(lo_options['PT'],'options_'+'PT')],
              [elmt.collapse(lo_options['BM4'],'options_'+'BM4')],
              [elmt.collapse(lo_options['MU2'],'options_'+'MU2')],

              ]
    lo_EOS = lo_EOS+lo_EOS_collapes
    lo_EOS = lo_EOS + [[elmt.Bc('fit EOS parameters','run_eos_fitting',('white', 'green')),elmt.Bc('plot fiting', 'PlotfittingEOS',('white', 'green'))], ]

    lo_poisson = [[elmt.T('nu:', 'nu'),elmt.dI('','nu',6), elmt.Bc('calculate','calc_nu',('white', 'green'),right_click_menu=['',['details...::nu']])]]

    lo_el = [[elmt.Chk('parameters:','el',disabled=True),elmt.dI('','p_el',20),elmt.Bc('calculate','calc_el',('gray','gray'))]]

    lo_def = [[elmt.Chk('','def',disabled=True),elmt.T('Evac:','p_evac'),elmt.dI('','p_evac',3),elmt.T('Svac:','p_svac'),elmt.dI('','p_svac',3),elmt.T('Tm:','Tm'),elmt.dI('','Tm',7)]]

    lo_anhxc = [[elmt.Chk('parameters:','anhxc',disabled=True),elmt.dI('','p_anhxc',10)]]

    lo_intanh = [[elmt.Chk('parameters:','intanh',disabled=True),elmt.dI('','p_intanh',10)]]

    lo_left = [[elmt.T('path for input files:','compound'),
                elmt.dI('','compound',30),
                elmt.dI(txt='', key='FILEBROWSE_', w=0, disabled=False,enable_events=True,visible=False),
                elmt.Br('compound')],
               [elmt.T('formula:','formula'),elmt.dI('','formula',6),elmt.T('structure:','strkt'),elmt.dI('','strkt',4)],
               [elmt.T('mass: (Kg/mol-at)','mass'),elmt.dI('','mass',7),elmt.T('r:','r',visible=False),elmt.dI('1','r',5,visible=False)],
               [elmt.F('EOS parametrization',lo_EOS,'EOS', right_click_menu=['',['More Info...::EOS','Go to code...::GoToEOS',]])],
               [elmt.F("Poisson's ratio", lo_poisson,'poisson', right_click_menu=['',['More Info...::Poisson','Go to code...::GoToPoisson',]])],
               [elmt.F('Electronic contribution',lo_el,'el', right_click_menu=['',['More Info...::Electronic','Go to code...::GoToElectronic',]])],
               [elmt.F('Defects (mono-vacancies)',lo_def, 'def', right_click_menu=['',['More Info...::Defects','Go to code...::GoToDefects',]])],
               [elmt.F('Anharmonicity XC',lo_anhxc, 'anhxc', right_click_menu=['','Not Implemented...::Anh',])],
               [elmt.F('Intrinsic Anharmonicity',lo_intanh, 'intanh', right_click_menu=['','Not Implemented...::Intanh',])],
               ]

    lo_minF = [[elmt.T('P (Pa):','Pi'),elmt.dI('','Pi',4),elmt.T('initial T (K):','Ti'),elmt.dI('','Ti',4),elmt.T('final T (K):','Tf'),elmt.dI('','Tf',4),elmt.T('No. steps:','ntemps'),elmt.dI('','ntemps',4)],
               [elmt.Chk('Slater (lam=-1)', 'mode_jjsl'), elmt.Chk('Dugdale-MacDonald (lam=0)', 'mode_jjdm'),elmt.Chk('Free Volume (lam=1)', 'mode_jjfv')],
               # [elmt.Chk('Dugdale–McDonald', 'mode_DM'),elmt.Chk('Slater', 'mode_Sl'),elmt.Chk('Vaschenko–Zubarev', 'mode_VZ'),elmt.Chk('Free volume', 'mode_mfv')],
               [elmt.Bc('run minimization','run_minF',('gray','gray'))],
               [elmt.sCol([[elmt.M('','minF_output',100,7)]], 'minF_output', 480, 80)],
               [elmt.Bc('Plot V(T)','plotter',('gray','gray'))],]

    lo_tabs_tprops = [[elmt.Tab(eos_str,[[elmt.sCol([[elmt.M('','tprop_'+eos_str,1000,7)]], 'tprop_'+eos_str, 470, 80)]],eos_str,False) for eos_str in ['','MP','BM','RV','MG','TB','MU','PT','BM4','MU2','EAM','*MP','*BM','*RV','*MG','*TB','*MU','*PT','*BM4','*MU2','*EAM']]]

    lo_tprops = [[elmt.Bc('evaluate','eval_tprops',('gray','gray')), elmt.Bc('evaluate anh','eval_anh',('gray','gray'))],
             [elmt.TG(lo_tabs_tprops,'tabs_tprops')],
             [elmt.T('select property to plot:','prop2plt'),elmt.ICombo(['       ','       ','       ','       '],'prop2plt',10,1),elmt.Bc('Plot','plotter_tprops',('white',elmt.theme_background_color()))]]

    lo_tabs_fsparams = [[elmt.Tab(k,lo_fsparams(k),'fs_'+k,False) for k in ['','MP','BM','RV','MG','TB','MU','PT','BM4','MU2','EAM','*MP','*BM','*RV','*MG','*TB','*MU','*PT','*BM4','*MU2','*EAM']]]

    lo_collaps_fsparams = [elmt.TG(lo_tabs_fsparams,'tabs_fsparas')]

    lo_right = [[elmt.F('Temperature dependence of equilibrium volume, V(T)',lo_minF,'minF')],
                [elmt.F('Thermodynamic Properties',lo_tprops,'tprops')],
                [elmt.F('FactSage Parameters',[[elmt.T('from T=','fs_Tfrom'),elmt.Ipad('','fs_Tfrom',4,pad=(0,0)),
                                                elmt.T('to T=','fs_Tto'),elmt.Ipad('','fs_Tto',4,pad=(0,0)),elmt.Bc('Run parametrization', 'run_fs_params',('gray','gray'))],
                                               lo_collaps_fsparams],'fsparams')],]
    layout = [[elmt.C(lo=lo_left,key='left'),
               elmt.VS(),
               elmt.C(lo=lo_right, key='right')],]

    return layout

def layout_bulk(initial_folder):
    lo_contr = [[elmt.Chk('electronic','el_blk'),elmt.Chk('defects','def_blk')],
                [elmt.Chk('anharmonic correction','anh_blk'),elmt.Chk('intrinsic anharmonicity','intanh_blk')],
                ]
    return [[elmt.T('folder:','folder_bulk'),
                elmt.I('','folder_bulk',10),
                elmt.dI(txt='', key='FILEBROWSE_bulk_', w=0, disabled=False,enable_events=True,visible=False),
                elmt.Br('folder_bulk',initial_folder)],
            [elmt.listbox(['BM','RV','MG','TB','MU','PT','BM4','MU2','EAM'],'bulk'),elmt.listbox([],'bulk_list')],
            [elmt.F('Contributions',lo_contr,'contr_bulk')],
            [elmt.Bc('Run','run_eos_fitting',('grey','grey'))],
            ]



from PySide6.QtWidgets import QMainWindow, QWidget, QVBoxLayout
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.widgets import Cursor
from matplotlib.figure import Figure
import numpy as np


# Main window
class windowPlot(QMainWindow):
    def __init__(self, parent=None):
        # Initialize QMainWindow
        super().__init__()
        # Get MplWidget
        self.mpl_widget = MplWidget()
        # Set MplWidget as central widget
        self.setCentralWidget(self.mpl_widget)
        # Create axes
        self.ax = self.mpl_widget.canvas.figure.add_subplot(111)
        # Reference Cursor
        self.cursor = Cursor(self.ax, useblit=True, linewidth=0.8, color='lightgray')
        # Plot some random stuff
#        x, y = np.random.rand(100), np.random.rand(100)
#        self.ax.scatter(x, y)
#        # Show app
#        self.show()


# MplCanvas
class MplWidget(QWidget):
    def __init__(self, parent=None):
        # Initialize QWidget
        super().__init__(parent)
        # Create canvas
        self.canvas = FigureCanvas(Figure())
        # Set constrained layout
        self.canvas.figure.set_constrained_layout(True)
        # Create layout
        layout = QVBoxLayout()
        # Add canvas
        layout.addWidget(self.canvas)
        # Set layout
        self.setLayout(layout)


## Initialize app
#app = QApplication([])
#UI = MainWindow()
#app.exec()



import uuid
import re
def keygen():
    mac_lst = re.findall('..', '%012x' % uuid.getnode())
    key_lst = []
    A = 'abcdEF123456gHIJklMNop556372834q6rs6TU7vWxyzAbcDEfgHiJklMNoPQrsTUVwxYZabcdEFgHIJklMNopqrsTUvWxyzAbcDEfgHiJklMNoPQrsTUVwxYZ'
    B = 'Abc123DEf5g6H7i8J8k9l0MNoPQrsTUVwxYZAbc7D6E5f4g3H2i2J2klMNoPQrsTUVwxYZabcdEFgHIJklMNopqrsTUvWxyzabcdEFgHIJklMNopqrsTUvWxyz'
    for l in mac_lst:
        # print(l.isnumeric())
        if  l.isnumeric() is not True:
            k = ''.join(format(ord(i), '08b') for i in l)
        else:
            a=int(l[0])
            b=int(l[1])
            c= str(a*b)
            k = ''.join(format(ord(i), '08b') for i in c)
        for i, a in enumerate(k):
            if int(a) == 0:
                key_lst.append(A[i])
            if int(a) == 1:
                key_lst.append(B[i])
    key = ''.join(key_lst)
    return key



from PySide6.QtWidgets import  QDialog, QFileDialog
from debyetools.tpropsgui.ui_dialog_doscar import Ui_Dialog as Ui_DOSCAR


class dialogDOSCAR(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_DOSCAR()
        self.ui.setupUi(self)

        self.ui.browse.clicked.connect(self.getfiles)
        self.ui.ok.clicked.connect(self.on_pushButton_OK)
        self.filepath='.'

    def getfiles(self):
            filepath = QFileDialog.getExistingDirectory(self, caption='Select a folder')
            self.ui.filepath.setText(filepath)

    def on_pushButton_OK(self):
        self.filepath = self.ui.filepath.text()
        self.close()



# import re
import numpy as np
import debyetools.tpropsgui.toolbox as tbox
import debyetools.tpropsgui.plotter as plot
from debyetools.fs_compound_db import Cp2fit, alpha2fit, Ksinv2fit, Ksp2fit
#
def fbrowser_fill_browser(window,event):
    str_I_compound = window['--I_compound'].get()
    str_folderbrowser = window[event].get()
    if str_folderbrowser == '': str_folderbrowser = str_I_compound

    window['--I_compound'].update(str_folderbrowser)
    window['--I_compound'].update(move_cursor_to='end')
    return str_folderbrowser
#
def fbrowser_update_fields(window,window7,contcar_str,mws_dict,str_folderbrowser,opened_EOS_dict,EOS_long_lst,EOS_str_lst,checked_EOS_dict,bool_anh):

    with open(window['--I_compound'].get() +contcar_str) as f:
        lines = f.readlines()
    els_lst = lines[5].split()
    nats_lst = [int(i) for i in lines[6].split()]
    nat = np.sum(nats_lst)

    mass = 0
    for ei, ni in zip(els_lst, nats_lst):
        mass+=mws_dict[ei]*ni

    nats_lst_int = [int(i) for i in np.array(nats_lst)/tbox.compute_gcd(nats_lst)]

    window['--I_formula'].update(''.join(['%s%s' for i in els_lst])%tuple(np.reshape(list(zip(els_lst, nats_lst_int)),len(nats_lst)*2)))
    window['--I_mass'].update(mass/nat)
    window['--I_strkt'].update(str_folderbrowser.split('_')[-1])
    window['--I_p_el'].update('0, 0, 0, 0')
    window['--I_p_intanh'].update('0, 1')
    window['--I_p_anhxc'].update('0, 0, 0')
    window['--I_p_evac'].update('8')
    window['--I_p_svac'].update('2')
    window['--I_Tm'].update('1000')
    window['--I_Ti'].update('0.1')
    window['--I_Pi'].update('0')
    window['--I_Tf'].update(window['--I_Tm'].get())
    window['--I_ntemps'].update(10)
    window['--I_Tm'].update(disabled=False)
    window['--I_mass'].update(disabled=False)
    window['--M_minF_output'].update('')
    window['--LBx_EOS_listbox'].update(disabled=False)
    window['--LBx_EOS_listbox'].set_value([False for k in window['--LBx_EOS_listbox'].get()])
    add_EOS(window, opened_EOS_dict,EOS_long_lst)
    for eos in EOS_str_lst:
        window['--I_params_'+eos].update('0, 0, 0, 0')
    window['--I_nu'].update('')
    window['--I_p_el'].update('0, 0, 0, 0')
    window['--I_p_evac'].update('8')
    window['--I_p_svac'].update('2')
    window['--I_Tm'].update('1000')
    window['--I_p_intanh'].update('0, 1')
    window['--Chk_el'].update(False)
    window['--I_p_el'].update(disabled= not bool(window['--Chk_el'].get()))
    window['||B_calc_el'].update(disabled= not bool(window['--Chk_el'].get()))
    window['--Chk_def'].update(False)
    window['--I_p_evac'].update(disabled= not bool(window['--Chk_def'].get()))
    window['--I_p_svac'].update(disabled= not bool(window['--Chk_def'].get()))
    window['--I_Tm'].update(disabled= not bool(window['--Chk_def'].get()))
    window['--Chk_intanh'].update(False)
    window['--I_p_intanh'].update(disabled= not bool(window['--Chk_intanh'].get()))
    window['--Chk_anhxc'].update(False)
    window['--I_p_anhxc'].update(disabled= not bool(window['--Chk_anhxc'].get()))

    for k in EOS_str_lst:
        window['--M_tprop_'+k].update('')
        window['--Tab_'+k].update(visible=False)
        window['--Tab_fs_'+k].update(visible=False)
        window['--I_H298'+k].update('',disabled=True)
        window['--I_S298'+k].update('',disabled=True)
        for i in range(6):
            window['--I_fsCp_P'+str(i)+k].update('',disabled=True)
        for i in range(4):
            window['--I_fsa_P'+str(i)+k].update('',disabled=True)
        for i in range(4):
            window['--I_fsK_P'+str(i)+k].update('',disabled=True)
        for i in range(2):
            window['--I_fsKp_P'+str(i)+k].update('',disabled=True)
    window['--Tab_'].update(visible=True)
    window['--Tab_'].select()
    window['--Tab_fs_'].update(visible=True)
    window['--Tab_fs_'].select()

    window['--I_fs_Tfrom'].update('')
    window['--I_fs_Tfrom'].update(disabled = True)
    window['--I_fs_Tto'].update('')
    window['--I_fs_Tto'].update(disabled = True)
    window['||B_plotter'].update(disabled=True)
    window['||B_plotter_tprops'].update(disabled=True)
    window['||B_plotter_fsprop2plt'].update(disabled=True)
    window['||B_eval_tprops'].update(disabled=True)
    window['||B_eval_anh'].update(disabled=True)
    window['||B_run_fs_params'].update(disabled=True)

    window['--IC_prop2plt'].update('')

    checked_EOS_dict = update_diabled(window,window7,opened_EOS_dict,EOS_str_lst,checked_EOS_dict, bool_anh)
    return checked_EOS_dict

#
def chk_eos(window,opened_dict):
    for k in opened_dict.keys():
        window['||Col_options_'+k].update(visible=opened_dict[k])
#
def chk_calc_params(window,event):
    eos_str = event.split('_')[-1]
    window['--I_params_'+eos_str].update(disabled= bool(window[event].get()))
#
def bool_chks(window,opened_dict):
    bool_dict_params_EOS = {stri:window['--Chk_calc_params_'+stri].get() for stri in opened_dict.keys()}
    bool_dict_EOS =  {stri:opened_dict[stri] for stri in opened_dict.keys()}
    bool_run_eos_fitting = any([all([True,bool_dict_EOS[stri]]) for stri in opened_dict.keys()])
    bool_mode = any([window['--Chk_mode_'+stri].get() for stri in ['jjsl', 'jjdm', 'jjfv',
                                                                   # 'DM', 'Sl', 'VZ', 'mfv'
                                                                   ]])


    return bool_run_eos_fitting, bool_dict_params_EOS, bool_mode
def add_EOS(window, opened_EOS_dict,EOS_long_lst):
    for k in opened_EOS_dict.keys():
        opened_EOS_dict[k]=False
    for k in window['--LBx_EOS_listbox'].get():
        opened_EOS_dict[EOS_long_lst[k]]=True
    # print(opened_EOS_dict)

    chk_eos(window,opened_EOS_dict)

#
def update_diabled(window,window7,opened_dict,eos_available,bool_dict_params_EOS, bool_anh):
    bool_run_eos_fitting, bool_dict_params_EOS, bool_mode = bool_chks(window,opened_dict)
    # window['||B_run_eos_fitting'].update(disabled=not bool_run_eos_fitting)

    bool_minF = bool_run_eos_fitting and (True if window['--I_nu'].get()!='' else False) and bool_mode

    window['||B_run_minF'].update(disabled=not bool_minF)

    window['--I_nu'].update(disabled=True if window['--I_compound'].get()=='' else False)
    window['||B_calc_nu'].update(disabled=True if window['--I_compound'].get()=='' else False)
    window['--Chk_el'].update(disabled=True if window['--I_compound'].get()=='' else False)


    for str_eos in eos_available:
        window['||B_plotter_fsprop2plt'+str_eos].update(disabled=True if window['--IC_fsprop2plt'+str_eos].get()== '' else False)

    window['||B_add_EOS'].update(disabled=True if window['--I_compound'].get()=='' else False)
    window['||B_run_eos_fitting'].update(disabled=True if window['--I_compound'].get()=='' else False)
    window['||B_PlotfittingEOS'].update(disabled=True if window['--I_compound'].get()=='' else False)
    window['--Chk_def'].update(disabled=True if window['--I_compound'].get()=='' else False)
    window['--Chk_intanh'].update(disabled=True if window['--I_compound'].get()=='' else False)
    window['--Chk_anhxc'].update(disabled=True if window['--I_compound'].get()=='' else False)
    window['--I_Ti'].update(disabled=True if window['--I_compound'].get()=='' else False)
    window['--I_Pi'].update(disabled=True if window['--I_compound'].get()=='' else False)
    window['--I_Tf'].update(disabled=True if window['--I_compound'].get()=='' else False)
    #window['--I_Tm'].update(disabled=True if window['--I_compound'].get()=='' else False)
    window['--I_ntemps'].update(disabled=True if window['--I_compound'].get()=='' else False)
    window['||B_plotter_tprops'].update(disabled=True if window['--IC_prop2plt'].get()== '' else False)
    if bool_anh:
        window7['||B_plotter_anh'].update(disabled=True if window7['--IC_anh2plt'].get() == '' else False)
    else:
        pass
    window['--I_cutoff_MP'].update(disabled=False)
    window['--I_ndists_MP'].update(disabled=False)

    return bool_dict_params_EOS
#
def eos_write_params(window,EOSStr,pEOS):
        window['--I_params_'+EOSStr].update(', '.join(['%.9e' for i in pEOS])%tuple(pEOS))
        pEOS2=pEOS
        # if EOSStr not in ['EAM', 'MP']:
        #     pEOS2[0]=pEOS2[0]/(0.160218e-18 * 6.02214e23)
        #     pEOS2[1] = pEOS2[1] / (1e-30 * 6.02e23)
        # print(EOSStr,':',', '.join(['%.9e' for i in pEOS])%tuple(pEOS))
##
def chk_el(window,event):
    window['--I_p_el'].update(disabled= not bool(window[event].get()))
    window['||B_calc_el'].update(disabled= not bool(window[event].get()))
#
def chk_def(window,event):
    window['--I_p_evac'].update(disabled= not bool(window[event].get()))
    window['--I_p_svac'].update(disabled= not bool(window[event].get()))
    window['--I_Tm'].update(disabled= not bool(window[event].get()))
##
def chk_intanh(window,event):
    window['--I_p_intanh'].update(disabled= not bool(window[event].get()))
def chk_anhxc(window,event):
    window['--I_p_anhxc'].update(disabled= not bool(window[event].get()))
#
def minF_enable_nexts(window):
    window['||B_plotter'].update(disabled=False)
    window['||B_eval_tprops'].update(disabled=False)
    # window['||B_eval_anh'].update(disabled=False)
#
def tprops_enable_nexts(window):
    window['||B_run_fs_params'].update(disabled=False)
    window['--I_fs_Tfrom'].update('298.15')
    window['--I_fs_Tto'].update(window['--I_Tf'].get())
    window['--I_fs_Tfrom'].update(disabled=False)
    window['--I_fs_Tto'].update(disabled=False)
    window['||B_eval_anh'].update(disabled=False)
    window['--Tab_'].update(visible=False)
#
def plot_EvV(window, eosobj_dict, opened_EOS_dict,jx):
    pots_str_lst = [k for k in opened_EOS_dict if opened_EOS_dict[k]]
    # print(pots_str_lst)
    V_DFT = eosobj_dict['V_DFT']
    E_DFT = eosobj_dict['E_DFT']
    tab3_str='#V          DFT         '+'          '.join(['%s' for i in pots_str_lst])%tuple(pots_str_lst)+'\n'
    for Vi, Ei in zip(V_DFT, E_DFT):
        Emi = [eosobj_dict[k].E0(Vi) for k in pots_str_lst]
        tab3_str= tab3_str + '%.10e   %.10e  '%(Vi,Ei) + '  '.join(['%.10e' for i in Emi])%tuple(Emi)+'\n'
    # print(tab3_str)
    initial_tabs_multilinetxt = {'t0':{'multiline':tab3_str}}
    initial_lines_settings = {
                              'l0': {'plot':True,'label':0,'linestyle':'None','color':'mediumpurple',  'marker':'o',   'markerfacecolor':'black', 'markeredgecolor':'mediumpurple',  'linewidth':2,'markersize':10},
                              'l1': {'plot':True,'label':0,'linestyle':'None','color':'purple',        'marker':'+',   'markerfacecolor':'None', 'markeredgecolor':'purple',         'linewidth':2,'markersize':10},
                              'l2': {'plot':True,'label':0,'linestyle':'None','color':'gray',          'marker':'x',   'markerfacecolor':'None', 'markeredgecolor':'gray',           'linewidth':2,'markersize':10},
                              'l3': {'plot':True,'label':0,'linestyle':'None','color':'orchid',        'marker':'s',   'markerfacecolor':'None', 'markeredgecolor':'orchid',         'linewidth':2,'markersize':10},
                              'l4': {'plot':True,'label':0,'linestyle':'None','color':'deepskyblue',   'marker':'^',   'markerfacecolor':'None', 'markeredgecolor':'deepskyblue',    'linewidth':2,'markersize':10},
                              'l5': {'plot':True,'label':0,'linestyle':'None','color':'pink',          'marker':'>',   'markerfacecolor':'None', 'markeredgecolor':'pink',           'linewidth':2,'markersize':10},
                              'l6': {'plot':True,'label':0,'linestyle':'None','color':'aqua',          'marker':'1',   'markerfacecolor':'None', 'markeredgecolor':'aqua',           'linewidth':2,'markersize':10},
                              'l7': {'plot':True,'label':0,'linestyle':'None','color':'cornflowerblue','marker':'<',   'markerfacecolor':'None', 'markeredgecolor':'cornflowerblue', 'linewidth':2,'markersize':10},
                              'l8': {'plot':True,'label':0,'linestyle':'None','color':'C0',            'marker':'2',   'markerfacecolor':'None', 'markeredgecolor':'C0',             'linewidth':2,'markersize':10},
                              'l9': {'plot':True,'label':0,'linestyle':'None','color':'mediumpurple',  'marker':'.',   'markerfacecolor':'None', 'markeredgecolor':'mediumpurple',             'linewidth':2,'markersize':10},
                              'l11':{'plot':True,'label':0,'linestyle':'None','color':'purple',        'marker':'p',   'markerfacecolor':'None', 'markeredgecolor':'purple',             'linewidth':2,'markersize':10},
                              'l10':{'plot':True,'label':0,'linestyle':'None','color':'gray',          'marker':'4',   'markerfacecolor':'None', 'markeredgecolor':'gray',             'linewidth':2,'markersize':10},
                              'l12':{'plot':True,'label':0,'linestyle':'None','color':'orchid',        'marker':'d',   'markerfacecolor':'None', 'markeredgecolor':'orchid',             'linewidth':2,'markersize':10},
                              'l13':{'plot':True,'label':0,'linestyle':'None','color':'deepskyblue',   'marker':'+',   'markerfacecolor':'None', 'markeredgecolor':'deepskyblue',             'linewidth':2,'markersize':10},
                              'l14':{'plot':True,'label':0,'linestyle':'None','color':'pink',          'marker':'x',   'markerfacecolor':'None', 'markeredgecolor':'pink',             'linewidth':2,'markersize':10},
                              'l15':{'plot':True,'label':0,'linestyle':'None','color':'aqua',          'marker':'+',   'markerfacecolor':'None', 'markeredgecolor':'aqua',             'linewidth':2,'markersize':10},
                              }
    initial_fig_settings = {'figwidth':5.5,'figheight':4.5,'use_title':False,'title':'','titlexpos':.7,'titleypos':.9,
                            'titlesize':12,'use_xlabel':True,'use_ylabel':True,'xlabel':r'Volume $\left[\frac{m^3}{mol-at}\right]$','ylabel':r'$E~\left[\frac{J}{mol-at}\right]$','labelxsize':13,
                            'labelysize':13,'auto_xlim':True,'auto_ylim':True,'limxmin':-0.5,'limxmax':110,'limymin':-1,'limymax':2,'use_legend':True,'legend_loc':'best',
                            'legendncol':2,'legendfontsize':14,'use_grid':True,'lmargin':0.2,'rmargin':0.98,'tmargin':0.95,'bmargin':0.12, 'scalex':1,'scaley':1}

    data4plot = plot.pop_window(initial_tabs_multilinetxt,initial_lines_settings,initial_fig_settings,jx)

    return data4plot

def plot_EvV_full(window, eosobj_dict, opened_EOS_dict,jx):
    pots_str_lst = [k for k in opened_EOS_dict if opened_EOS_dict[k]]
    # print(pots_str_lst)
    V_DFT = eosobj_dict['V_DFT']
    E_DFT = eosobj_dict['E_DFT']
    tab3_str='#V          DFT         '+'          '.join(['%s' for i in pots_str_lst])%tuple(pots_str_lst)+'\n'
    for Vi, Ei in zip(V_DFT, E_DFT):
        Emi = [eosobj_dict[k].E0(Vi) for k in pots_str_lst]
        tab3_str= tab3_str + '%.10e   %.10e  '%(Vi,Ei) + '  '.join(['%.10e' for i in Emi])%tuple(Emi)+'\n'
    # print(tab3_str)
    initial_tabs_multilinetxt = {'t0':{'multiline':tab3_str}}
    initial_lines_settings = {
                              'l0': {'plot':True,'label':0,'linestyle':'None','color':'mediumpurple',  'marker':'o',   'markerfacecolor':'black', 'markeredgecolor':'mediumpurple',  'linewidth':2,'markersize':10},
                              'l1': {'plot':True,'label':0,'linestyle':'None','color':'purple',        'marker':'+',   'markerfacecolor':'None', 'markeredgecolor':'purple',         'linewidth':2,'markersize':10},
                              'l2': {'plot':True,'label':0,'linestyle':'None','color':'gray',          'marker':'x',   'markerfacecolor':'None', 'markeredgecolor':'gray',           'linewidth':2,'markersize':10},
                              'l3': {'plot':True,'label':0,'linestyle':'None','color':'orchid',        'marker':'s',   'markerfacecolor':'None', 'markeredgecolor':'orchid',         'linewidth':2,'markersize':10},
                              'l4': {'plot':True,'label':0,'linestyle':'None','color':'deepskyblue',   'marker':'^',   'markerfacecolor':'None', 'markeredgecolor':'deepskyblue',    'linewidth':2,'markersize':10},
                              'l5': {'plot':True,'label':0,'linestyle':'None','color':'pink',          'marker':'>',   'markerfacecolor':'None', 'markeredgecolor':'pink',           'linewidth':2,'markersize':10},
                              'l6': {'plot':True,'label':0,'linestyle':'None','color':'aqua',          'marker':'1',   'markerfacecolor':'None', 'markeredgecolor':'aqua',           'linewidth':2,'markersize':10},
                              'l7': {'plot':True,'label':0,'linestyle':'None','color':'cornflowerblue','marker':'<',   'markerfacecolor':'None', 'markeredgecolor':'cornflowerblue', 'linewidth':2,'markersize':10},
                              'l8': {'plot':True,'label':0,'linestyle':'None','color':'C0',            'marker':'2',   'markerfacecolor':'None', 'markeredgecolor':'C0',             'linewidth':2,'markersize':10},
                              'l9': {'plot':True,'label':0,'linestyle':'None','color':'mediumpurple',  'marker':'.',   'markerfacecolor':'None', 'markeredgecolor':'mediumpurple',             'linewidth':2,'markersize':10},
                              'l11':{'plot':True,'label':0,'linestyle':'None','color':'purple',        'marker':'p',   'markerfacecolor':'None', 'markeredgecolor':'purple',             'linewidth':2,'markersize':10},
                              'l10':{'plot':True,'label':0,'linestyle':'None','color':'gray',          'marker':'4',   'markerfacecolor':'None', 'markeredgecolor':'gray',             'linewidth':2,'markersize':10},
                              'l12':{'plot':True,'label':0,'linestyle':'None','color':'orchid',        'marker':'d',   'markerfacecolor':'None', 'markeredgecolor':'orchid',             'linewidth':2,'markersize':10},
                              'l13':{'plot':True,'label':0,'linestyle':'None','color':'deepskyblue',   'marker':'+',   'markerfacecolor':'None', 'markeredgecolor':'deepskyblue',             'linewidth':2,'markersize':10},
                              'l14':{'plot':True,'label':0,'linestyle':'None','color':'pink',          'marker':'x',   'markerfacecolor':'None', 'markeredgecolor':'pink',             'linewidth':2,'markersize':10},
                              'l15':{'plot':True,'label':0,'linestyle':'None','color':'aqua',          'marker':'+',   'markerfacecolor':'None', 'markeredgecolor':'aqua',             'linewidth':2,'markersize':10},
                              }
    initial_fig_settings = {'figwidth':5.5,'figheight':4.5,'use_title':False,'title':'','titlexpos':.7,'titleypos':.9,
                            'titlesize':12,'use_xlabel':True,'use_ylabel':True,'xlabel':'Volume $\left[m^3/mol-at\\right]$','ylabel':'$E~\left[J/mol-at\\right]$','labelxsize':13,
                            'labelysize':13,'auto_xlim':True,'auto_ylim':True,'limxmin':-0.5,'limxmax':110,'limymin':-1,'limymax':2,'use_legend':True,'legend_loc':'best',
                            'legendncol':2,'legendfontsize':14,'use_grid':True,'lmargin':0.2,'rmargin':0.98,'tmargin':0.95,'bmargin':0.12, 'scalex':1,'scaley':1}

    data4plot = plot.pop_window(initial_tabs_multilinetxt,initial_lines_settings,initial_fig_settings,jx)

    return data4plot

def plot_VvT(window,jx):
    initial_tabs_multilinetxt = {'t0':{'multiline':window['--M_minF_output'].get()}}
    initial_lines_settings = {
                              'l0':{'plot':True,'label':0,'linestyle':'-','color':'mediumpurple','marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'mediumpurple','linewidth':2,'markersize':10},
                              'l1':{'plot':True,'label':0,'linestyle':'-','color':'purple', 'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'deepskyblue','linewidth':2,'markersize':10},
                              'l2':{'plot':True,'label':0,'linestyle':'-','color':'gray',        'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'aqua','linewidth':2,'markersize':10},
                              'l3':{'plot':True,'label':0,'linestyle':'-','color':'orchid',        'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'gray','linewidth':2,'markersize':10},
                              'l4':{'plot':True,'label':0,'linestyle':'-','color':'deepskyblue',          'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'C0','linewidth':2,'markersize':10},
                              'l5':{'plot':True,'label':0,'linestyle':'-','color':'pink',          'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'C3','linewidth':2,'markersize':10},
                              'l6':{'plot':True,'label':0,'linestyle':'-','color':'aqua',      'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'orange','linewidth':2,'markersize':10},
                              'l7':{'plot':True,'label':0,'linestyle':'-','color':'cornflowerblue',        'marker':'None','markerfacecolor':'None', 'markeredgecolor':'None','linewidth':2,'markersize':10},
                              'l8':{'plot':True,'label':0,'linestyle':'-','color':'C0',        'marker':'None','markerfacecolor':'None', 'markeredgecolor':'None','linewidth':2,'markersize':10},
                              'l9':{'plot':True,'label':0,'linestyle':'-','color':'C1',        'marker':'None','markerfacecolor':'None', 'markeredgecolor':'None','linewidth':2,'markersize':10},
                              }

    initial_fig_settings = {'figwidth':5.5,'figheight':4.5,'use_title':False,'title':'','titlexpos':.7,'titleypos':.9,
                            'titlesize':12,'use_xlabel':True,'use_ylabel':True,'xlabel':'T $\left[K\\right]$','ylabel':'$V~\left[m^3/atom\\right]$','labelxsize':13,
                            'labelysize':13,'auto_xlim':True,'auto_ylim':True,'limxmin':-0.5,'limxmax':110,'limymin':-1,'limymax':2,'use_legend':True,'legend_loc':'best',
                            'legendncol':2,'legendfontsize':14,'use_grid':True,'lmargin':0.14,'rmargin':0.98,'tmargin':0.95,'bmargin':0.12, 'scalex':1,'scaley':1}

    data4plot = plot.pop_window(initial_tabs_multilinetxt,initial_lines_settings,initial_fig_settings,jx)

    return data4plot
#
def plot_tprops(window,minF_header,jx):
    initial_tabs_multilinetxt = {'t0':{'multiline':[]}}
    for ix, k in enumerate(minF_header):
        window['--M_tprop_'+str(k)].get()
        datas_dict = tbox.txt2dict(window['--M_tprop_'+str(k)].get())
        txt2M = '#T    '+k+'\n'
        for valsi in np.c_[datas_dict['T'].T,datas_dict[window['--IC_prop2plt'].get()].T]:
            txt2M = txt2M + '%.9e %.9e'%tuple(valsi)+'\n'
        initial_tabs_multilinetxt['t'+str(ix)]={'multiline':txt2M}

    initial_lines_settings = {
                              'l0':{'plot':True,'label':0,'linestyle':'-','color':'mediumpurple','marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'mediumpurple','linewidth':2,'markersize':10},
                              'l1':{'plot':True,'label':0,'linestyle':'-','color':'purple', 'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'deepskyblue','linewidth':2,'markersize':10},
                              'l2':{'plot':True,'label':0,'linestyle':'-','color':'gray',        'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'aqua','linewidth':2,'markersize':10},
                              'l3':{'plot':True,'label':0,'linestyle':'-','color':'orchid',        'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'gray','linewidth':2,'markersize':10},
                              'l4':{'plot':True,'label':0,'linestyle':'-','color':'deepskyblue',          'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'C0','linewidth':2,'markersize':10},
                              'l5':{'plot':True,'label':0,'linestyle':'-','color':'pink',          'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'C3','linewidth':2,'markersize':10},
                              'l6':{'plot':True,'label':0,'linestyle':'-','color':'aqua',      'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'orange','linewidth':2,'markersize':10},
                              'l7':{'plot':True,'label':0,'linestyle':'-','color':'cornflowerblue',        'marker':'None','markerfacecolor':'None', 'markeredgecolor':'None','linewidth':2,'markersize':10},
                              'l8':{'plot':True,'label':0,'linestyle':'-','color':'C0',        'marker':'None','markerfacecolor':'None', 'markeredgecolor':'None','linewidth':2,'markersize':10},
                              'l9':{'plot':True,'label':0,'linestyle':'-','color':'C1',        'marker':'None','markerfacecolor':'None', 'markeredgecolor':'None','linewidth':2,'markersize':10},
                              }
    initial_fig_settings = {'figwidth':5.5,'figheight':4.5,'use_title':False,'title':'','titlexpos':.7,'titleypos':.9,
                            'titlesize':12,'use_xlabel':True,'use_ylabel':True,'xlabel':'T $\left[K\\right]$','ylabel':window['--IC_prop2plt'].get(),'labelxsize':13,
                            'labelysize':13,'auto_xlim':True,'auto_ylim':True,'limxmin':-0.5,'limxmax':110,'limymin':-1,'limymax':2,'use_legend':True,'legend_loc':'best',
                            'legendncol':2,'legendfontsize':14,'use_grid':True,'lmargin':0.14,'rmargin':0.98,'tmargin':0.95,'bmargin':0.12, 'scalex':1,'scaley':1}
    # plot.pop_window_simple(initial_tabs_multilinetxt,initial_lines_settings,initial_fig_settings)
    data4plot = plot.pop_window(initial_tabs_multilinetxt,initial_lines_settings,initial_fig_settings,jx)
    return data4plot

def plot_anh(window,minF_header,jx):
    initial_tabs_multilinetxt = {'t0':{'multiline':[]}}
    for ix, k in enumerate(minF_header):
        window['--M_anh_'+str(k)].get()
        datas_dict = tbox.txt2dict(window['--M_anh_'+str(k)].get())
        txt2M = '#T    '+k+'\n'
        for valsi in np.c_[datas_dict['T'].T,datas_dict[window['--IC_anh2plt'].get()].T]:
            txt2M = txt2M + '%.9e %.9e'%tuple(valsi)+'\n'
        initial_tabs_multilinetxt['t'+str(ix)]={'multiline':txt2M}

    initial_lines_settings = {
                              'l0':{'plot':True,'label':0,'linestyle':'-','color':'mediumpurple','marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'mediumpurple','linewidth':2,'markersize':10},
                              'l1':{'plot':True,'label':0,'linestyle':'-','color':'purple', 'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'deepskyblue','linewidth':2,'markersize':10},
                              'l2':{'plot':True,'label':0,'linestyle':'-','color':'gray',        'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'aqua','linewidth':2,'markersize':10},
                              'l3':{'plot':True,'label':0,'linestyle':'-','color':'orchid',        'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'gray','linewidth':2,'markersize':10},
                              'l4':{'plot':True,'label':0,'linestyle':'-','color':'deepskyblue',          'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'C0','linewidth':2,'markersize':10},
                              'l5':{'plot':True,'label':0,'linestyle':'-','color':'pink',          'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'C3','linewidth':2,'markersize':10},
                              'l6':{'plot':True,'label':0,'linestyle':'-','color':'aqua',      'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'orange','linewidth':2,'markersize':10},
                              'l7':{'plot':True,'label':0,'linestyle':'-','color':'cornflowerblue',        'marker':'None','markerfacecolor':'None', 'markeredgecolor':'None','linewidth':2,'markersize':10},
                              'l8':{'plot':True,'label':0,'linestyle':'-','color':'C0',        'marker':'None','markerfacecolor':'None', 'markeredgecolor':'None','linewidth':2,'markersize':10},
                              'l9':{'plot':True,'label':0,'linestyle':'-','color':'C1',        'marker':'None','markerfacecolor':'None', 'markeredgecolor':'None','linewidth':2,'markersize':10},
                              }
    initial_fig_settings = {'figwidth':5.5,'figheight':4.5,'use_title':False,'title':'','titlexpos':.7,'titleypos':.9,
                            'titlesize':12,'use_xlabel':True,'use_ylabel':True,'xlabel':'T $\left[K\\right]$','ylabel':window['--IC_anh2plt'].get()+'$_{EOS}$'+'$-$'+window['--IC_anh2plt'].get()+'$_{Murnaghan}$','labelxsize':13,
                            'labelysize':13,'auto_xlim':True,'auto_ylim':True,'limxmin':-0.5,'limxmax':110,'limymin':-1,'limymax':2,'use_legend':True,'legend_loc':'best',
                            'legendncol':2,'legendfontsize':14,'use_grid':True,'lmargin':0.14,'rmargin':0.98,'tmargin':0.95,'bmargin':0.12, 'scalex':1,'scaley':1}
    # plot.pop_window_simple(initial_tabs_multilinetxt,initial_lines_settings,initial_fig_settings)
    data4plot = plot.pop_window(initial_tabs_multilinetxt,initial_lines_settings,initial_fig_settings,jx)
    return data4plot
#
def plot_fsprops(window,event,fs_params,Tfrom,Tto,TPs_calculated_dict,jx):
    str_eos = event.replace('||B_plotter_fsprop2plt','')
    fs_params_Cp = fs_params[str_eos]['Cp']
    fs_params_alpha = fs_params[str_eos]['a']
    fs_params_Ksinv = fs_params[str_eos]['1/Ks']
    fs_params_Ksp = fs_params[str_eos]['Ksp']
    T_data = TPs_calculated_dict[str_eos]['T']
    ix_Tfrom = np.where(np.round(TPs_calculated_dict[str_eos]['T'],2) == np.round(Tfrom,2))[0][0]
    ix_Tto = np.where(np.round(TPs_calculated_dict[str_eos]['T'],2) == np.round(Tto,2))[0][0]
    if window['--IC_fsprop2plt'+str_eos].get()=='Cp':
        prop2plt = [Cp2fit(Ti,fs_params_Cp[0],fs_params_Cp[1],fs_params_Cp[2],fs_params_Cp[3]) for Ti in T_data[ix_Tfrom:ix_Tto+1]] #,fs_params_Cp[4],fs_params_Cp[5]
        prop_data = TPs_calculated_dict[str_eos]['Cp']
        txt1 = '#T $C_P=P_0T^0+P_1T^1+P_2T^{-2}+P_3T^2+P_4T^{-.5}+P_5T^{-3}$\n'
        txt2 = '#T $C_P$\n'
    if window['--IC_fsprop2plt'+str_eos].get()=='alpha':
        prop2plt = [alpha2fit(Ti,fs_params_alpha[0],fs_params_alpha[1],fs_params_alpha[2],fs_params_alpha[3]) for Ti in T_data[ix_Tfrom:ix_Tto+1]]
        prop_data = TPs_calculated_dict[str_eos]['a']
        txt1 = '#T $alpha=Q_0T^0+Q_1T^1+Q_2T^{-1}+Q_3T^{-2}$\n'
        txt2 = '#T $alpha$\n'
    if window['--IC_fsprop2plt'+str_eos].get()=='1/K':
        prop2plt = [Ksinv2fit(Ti,fs_params_Ksinv[0],fs_params_Ksinv[1],fs_params_Ksinv[2],fs_params_Ksinv[3]) for Ti in T_data[ix_Tfrom:ix_Tto+1]]
        prop_data = [1/ks for ks in TPs_calculated_dict[str_eos]['Ks']]
        txt1 = '#T $1/Ks=R_0T^0+R_1T^1+R_2T^2+R_3T^3$\n'
        txt2 = '#T $1/Ks$\n'
    if window['--IC_fsprop2plt'+str_eos].get()=='dK/dP':
        prop2plt = [Ksp2fit(Ti,fs_params_Ksp[0],fs_params_Ksp[1]) for Ti in T_data[ix_Tfrom:ix_Tto+1]]
        prop_data = TPs_calculated_dict[str_eos]['Ksp']
        txt1 = '#T $dK/dP=S_0+S_1\cdot(T-298.15K)\ln(T/298.15K)$\n'
        txt2 = '#T $dK/dP$\n'

    initial_tabs_multilinetxt = {'t0':{'multiline':[]}}
    for Cpi,Ti in zip(prop2plt, T_data[ix_Tfrom:ix_Tto+1]):
        txt1 = txt1 + '%.9e %.9e'%(Ti, Cpi)+'\n'
    initial_tabs_multilinetxt['t1']={'multiline':txt1}

    for Cpi,Ti in zip(prop_data, T_data):
        txt2 = txt2 + '%.9e %.9e'%(Ti, Cpi)+'\n'
    initial_tabs_multilinetxt['t0']={'multiline':txt2}
    initial_lines_settings = {
                              'l0':{'plot':True,'label':0,'linestyle':'-','color':'mediumpurple','marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'mediumpurple','linewidth':2,'markersize':10},
                              'l1':{'plot':True,'label':0,'linestyle':'-','color':'purple', 'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'deepskyblue','linewidth':2,'markersize':10},
                              'l2':{'plot':True,'label':0,'linestyle':'-','color':'gray',        'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'aqua','linewidth':2,'markersize':10},
                              'l3':{'plot':True,'label':0,'linestyle':'-','color':'orchid',        'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'gray','linewidth':2,'markersize':10},
                              'l4':{'plot':True,'label':0,'linestyle':'-','color':'deepskyblue',          'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'C0','linewidth':2,'markersize':10},
                              'l5':{'plot':True,'label':0,'linestyle':'-','color':'pink',          'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'C3','linewidth':2,'markersize':10},
                              'l6':{'plot':True,'label':0,'linestyle':'-','color':'aqua',      'marker':'None',   'markerfacecolor':'None', 'markeredgecolor':'orange','linewidth':2,'markersize':10},
                              'l7':{'plot':True,'label':0,'linestyle':'-','color':'cornflowerblue',        'marker':'None','markerfacecolor':'None', 'markeredgecolor':'None','linewidth':2,'markersize':10},
                              'l8':{'plot':True,'label':0,'linestyle':'-','color':'C0',        'marker':'None','markerfacecolor':'None', 'markeredgecolor':'None','linewidth':2,'markersize':10},
                              'l9':{'plot':True,'label':0,'linestyle':'-','color':'C1',        'marker':'None','markerfacecolor':'None', 'markeredgecolor':'None','linewidth':2,'markersize':10},
                              }
    initial_fig_settings = {'figwidth':5.5,'figheight':4.5,'use_title':False,'title':'','titlexpos':.7,'titleypos':.9,
                            'titlesize':12,'use_xlabel':True,'use_ylabel':True,'xlabel':'T $\left[K\\right]$','ylabel':window['--IC_prop2plt'].get(),'labelxsize':13,
                            'labelysize':13,'auto_xlim':True,'auto_ylim':True,'limxmin':-0.5,'limxmax':110,'limymin':-1,'limymax':2,'use_legend':True,'legend_loc':'best',
                            'legendncol':1,'legendfontsize':11,'use_grid':True,'lmargin':0.14,'rmargin':0.98,'tmargin':0.95,'bmargin':0.12, 'scalex':1,'scaley':1}
    data4plot = plot.pop_window(initial_tabs_multilinetxt,initial_lines_settings,initial_fig_settings,jx)
    return data4plot

#



import itertools as it
import re
import numpy as np
from typing import Tuple

class logging(object):
    def __init__(self, *files: ...) -> None:
        self.files = files
    def write(self, obj: str) -> None:
        for f in self.files:
            f.write(obj)
            f.flush()
    def flush(self)  -> None:
        for f in self.files:
            f.flush()
def c_types(atom_types: str) -> Tuple[list, list]:
    """
    returns all the pair types combinations.

    :param str atom_types: the types of each atom in the primitive cell in the same order as the basis vectors.
    :return: pair types and list wuth individual types.
    :rtype: Tuple[list, list]
    """
    types_all = re.findall('[A-Z][^A-Z]*', atom_types)
    ptypes=list(set([s for s in types_all]))
    ptypes.sort()
    combs_types = list(it.combinations_with_replacement(ptypes, r=2))
    combs_types = [A[0]+'-'+A[1] for A in combs_types]
    return combs_types, types_all

def generate_cells_coordinates(size: np.ndarray, primitive_cell: np.ndarray, center: np.ndarray) -> np.ndarray:
    """ generate the cell coordinates for which we are going
    to calculate the neighbor list.

    :param np.ndarray size: Number of times we are replicating the primitive cell
    :param np.ndarray primitive_cell: The primitive cell
    :param np.ndarray center: The position in space where the system of reference is
    :return: cell coordinates.
    :rtype: np.ndarray

    """

    cell_coords = np.array(list((it.product(np.arange(size[0]),
                                            np.arange(size[1]),
                                            np.arange(size[2])))))

    cell_coords_centered = cell_coords + center
    cell_coords_centered = np.dot(cell_coords_centered, primitive_cell)

    return cell_coords_centered

def gen_Ts(Ti: float, Tf: float, nTs: int) -> np.ndarray:
    """
    Function to generate a range of temperatures.

    :param float Ti: Initial temperature. (Try not to use the value 0. Use 0.1 instead.)
    :param float Tf: Final temperature.
    :param int nTs: Number of values. This does not include room temperature, which is included anyways.

    :retun np.ndarray: Values of temperatures between Ti and Tf, inclusive, plus room temperature.
    """
    minF_step = (Tf - Ti)/(nTs - 1.)
    Ts = np.arange(Ti, Tf+minF_step, minF_step)
    Ts = np.r_[Ts, [298.15]]
    Ts.sort()
    return Ts
def gen_Ps(Pi,Pf,nPs):
    """
    Function to generate a range of pressures.

    :param float Pi: Initial pressure.
    :param float Pf: Final pressure.
    :param int nPs: Number of values. This does not include room pressure, which is included anyways.

    :retun: Values of pressures between Pi and Pf.
    :rtype: np.ndarray
    """
    if nPs <= 1: return np.array([Pi])
    minF_step = (Pf - Pi)/(nPs - 1.)
    Ps = np.arange(Pi, Pf+1, minF_step)

    return Ps

def load_doscar(filename_sufix: str, list_filetags: list = None) -> tuple[list,list,list]:
    """
    Extract electronic density, energies and Fermi level as function of volume from DOSCAR's.
    :param filename_sufix: folder path.
    :type filename_sufix: str
    :param list_filetags: filename tags.
    :type list_filetags: list
    :return: E,N,Ef.
    :rtype: tuple[list,list,list]
    """
    if list_filetags is None:
        list_filetags = ['-0.10', '-0.09','-0.08','-0.07','-0.06','-0.05','-0.04','-0.03',
                         '-0.02','-0.01','-0.00','0.01','0.02','0.03','0.04','0.05','0.06',
                         '0.07','0.08','0.09','0.10']

    list_filetags = [str(li) for li in list_filetags]
    E = []
    N = []
    Ef = []
    nat = 0
    for dosfile in list_filetags:
        countline = 0
        EN = []

        filename = filename_sufix+dosfile
        with open(filename) as infile:
                for line in infile:
                    if countline == 0:
                        nat = float(line.split()[0])
                    if countline == 5:
                        Ef.append(float(line.split()[3]))
                    if countline > 5:
                        EN.append(line.split()[0:2])

                    countline +=1
        ENAl = np.array(EN)
        # print(dosfile, EN)
        E.append([float(s) for s in list(ENAl[:,0])])
        N.append([float(s)/nat for s in list(ENAl[:,1])])

    return E,N,Ef

def load_V_E(energy_dir_summary: str, energy_dir_contcar: str, units: str = 'eV/atom') -> tuple[np.ndarray,np.ndarray]:
    """
    Loads Energy curve as function of volume from VASP outputs.
    :param energy_dir_summary: Summary file path.
    :type energy_dir_summary: str
    :param energy_dir_contcar: Atoms positions file path.
    :type energy_dir_contcar: str
    :param units: units.
    :type units: str
    :return: Energy as function of volume
    :rtype: tuple[np.ndarray,np.ndarray]
    """
    with open(energy_dir_contcar,'r') as f_poscar:
        f_poscar_lines = f_poscar.readlines()
        cell_poscar = f_poscar_lines[2:5]

        cell_lst = [c.split() for c in cell_poscar]
        diag_cell = [float(c[i]) for i, c in enumerate(cell_lst)]

        try:
            nat = sum([int(li) for li in np.fromstring(f_poscar_lines[5], dtype=int, sep=' ')])
            1/nat
        except:
            nat = sum([int(li) for li in np.fromstring(f_poscar_lines[6], dtype=int, sep=' ')])
            1/nat
    with open(energy_dir_summary) as f_summary:
        f_summary_lines = f_summary.readlines()
        f_summary_lines = list(dict.fromkeys(f_summary_lines))
        ds = []
        E = []
        for l in f_summary_lines:
            l_lst = l.split(' ')
            ds.append(float(l_lst[0]))
            E.append(float(l_lst[3])/nat)

    V = []
    for di in ds:
        V.append(np.product(np.array(diag_cell)*(1+di))/nat)

    uconvV, uconvE = None, None
    if units=='J/mol':
        uconvE=(0.160218e-18*6.02214e23)
        uconvV=(1e-30*6.02e23)
    elif units=='eV/atom':
        uconvE,uconvV = 1,1
    return np.array(V).T*uconvV, np.array(E).T*uconvE

def load_EM(filename_outcar_eps: str) -> np.ndarray:
    """
    Extract the stiffness tensor from the VASP output (OUTCAR for IBRION=6).
    :param filename_outcar_eps: file path.
    :type filename_outcar_eps: str
    :return: Stiffness tensor.
    :rtype: np.ndarray
    """
    EM = []

    with open(filename_outcar_eps) as f:
        lines = f.readlines()
        for i, line in enumerate(lines):
            if line.startswith('  SYMMETRIZED ELASTIC MODULI (kBar)'):
                j = i + 3
                data = lines[j:j+6]
                break

    for line in data:
        EM += [[float(x) for x in line.split()[1:]]]
    EM = np.array(EM)

    return EM

def load_cell(filename_contcar: str) -> tuple[str,np.ndarray,np.ndarray]:
    """
    Extract crystal structure from file in VASP format (POSCAR or CONTCAR).
    :param filename_contcar: File path
    :type filename_contcar: str
    :return: formula,, cell, and basis.
    :rtype: tuple[str,np.ndarray,np.ndarray]
    """
    with open(filename_contcar) as f:
        poscar_lines=f.readlines()
    mult = float(poscar_lines[1])
    cell = np.array([np.fromstring(line_i, dtype=float,sep=' ') for line_i in poscar_lines[2:5]])
    cell = cell*mult

    ix_nats=6
    re.findall('[A-Z][^A-Z]*', 'ABC')
    ats_types = re.findall('[A-Z][^A-Z]*', poscar_lines[ix_nats-1].replace('  ','').replace(' ','').replace('\n',''))
    ats_types = [ai+'x' for ai in ats_types]
    nats = np.fromstring(poscar_lines[ix_nats], dtype=int,sep=' ')

    formula_lst=[]
    for at_i, na_i in zip(ats_types,nats):
        formula_lst.append(at_i*na_i)
    formula=''.join(formula_lst)
    tots_nats=sum(nats)

    basis = np.array([np.fromstring(line_i, dtype=float,sep=' ') for line_i in poscar_lines[8:8+tots_nats]])
    # basis = np.dot(basis,cell)

    return formula.replace('x', ''), cell, basis



from PySide6.QtWidgets import QMainWindow
from PySide6.QtGui import QIcon

from debyetools.tpropsgui.dialog_doscar import dialogDOSCAR
from debyetools.tpropsgui.dialog_summary import dialogSUMMARY
from debyetools.tpropsgui.dialog_outcar import dialogOUTCAR
from debyetools.tpropsgui.dialog_warning import dialogWarning
from debyetools.tpropsgui.window_cp import windowCp
from debyetools.tpropsgui.window_interatomicparams import windowInteratormic
from debyetools.tpropsgui.plot_EV import windowPlot

from debyetools.tpropsgui.ui_mainwindow import Ui_MainWindow as Ui_MW
from debyetools.tpropsgui.ui_missingkey import Ui_Form as Ui_KEY

from debyetools.tpropsgui.atomtools import Molecule, dt_potentials

import numpy as np

from debyetools.poisson import poisson_ratio as dt_poisson_ratio
from debyetools.aux_functions import load_doscar as dt_load_doscar
from debyetools.electronic import fit_electronic as dt_fit_electronic

from debyetools.tpropsgui.lock import keygen as kg

class MainWindow(QMainWindow):
    def __init__(self, parent=None, app=None):

        super().__init__(parent)

        key1 = kg()
        try:
            f = open('debyetools/tpropsgui/keydtgl','r')
            key0 = f.read().replace('\n','')
        except:
            key0=''
            passed=False
        if key0 != key1:
            passed = True
        else:
            passed = True

        if passed:
            self.ui = Ui_MW()
        else:
            self.ui = Ui_KEY()
            self.ui.setupUi(self)
            return

        self.ui.setupUi(self)
        my_icon = QIcon()
        my_icon.addFile('icon.ico')

        self.setWindowIcon(my_icon)

        self.cp_window = windowCp(self)
        self.cp_window.app = app
        self.doscardialog = dialogDOSCAR(self)
        self.summarydialog = dialogSUMMARY(self)
        self.outcardialog = dialogOUTCAR(self)
        self.warningdialog = dialogWarning(self)

        self.ui.pushButton.clicked.connect(self.on_pushButton_fitEOS)
        self.ui.pushButton_5.clicked.connect(self.on_pushButton_Cp)
        self.ui.pushButton_2.clicked.connect(self.on_pushButton_calc_nu)

        self.ui.pushButton_3.clicked.connect(self.on_pushButton_calculate_el)

        self.eos_str = 'BM'

        self.molecule = Molecule()

        self.ui.comboBox.currentIndexChanged.connect(self.selectionchange)


        self.check_el, self.check_def, self.check_anh, self.check_xs = self.ui.checkBox, self.ui.checkBox_2, self.ui.checkBox_3, self.ui.checkBox_4

        self.state_el = False
        self.state_def = False
        self.state_anh = False
        self.state_xs = False

        self.check_el.stateChanged.connect(self.on_check_el)
        self.check_def.stateChanged.connect(self.on_check_def)
        self.check_anh.stateChanged.connect(self.on_check_anh)
        self.check_xs.stateChanged.connect(self.on_check_xs)

        self.ui.progress.setGeometry(450, 525, 80, 5)
        self.ui.progress_2.setGeometry(260, 165, 80, 5)

        self.ipotparamsdialog = windowInteratormic(self)
        self.ipotparamsdialog.args = (None,)
        self.ipotparamsdialog.external_params_lineText = self.ui.lineEdit_2

        self.plotwindow = windowPlot(self)
        self.ui.actionEV.triggered.connect(self.plotEV)

        self.ui.actionDOSCAR.triggered.connect(self.on_pushButton_loaddata)
        self.ui.actionSUMMARY.triggered.connect(self.on_pushButton_loadSUMMARY)
        self.ui.actionOUTCAR.triggered.connect(self.on_pushButton_loadOUTCAR)

    def plotEV(self):
        Xdata = self.Vdata/1e-5
        Ydata = self.Edata/1e3
        self.plotwindow.ax.cla()
        self.plotwindow.ax.plot(Xdata, Ydata, ls = '', marker='.', label = 'data', mfc='None', mec='black', markersize=10,mew=1)
        self.plotwindow.ax.plot(Xdata, [self.eos.E0(Vi)/1e3 for Vi in Xdata*1e-5], ls = '', marker='+', label = 'model', mfc='purple', mec='purple', markersize=10,mew=1)

        spanX = (max(Xdata) - min(Xdata))/20
        spanY = (max(Ydata) - min(Ydata))/20
        self.plotwindow.ax.set_xlim((min(Xdata)-spanX, max(Xdata)+spanX))
        self.plotwindow.ax.set_ylim((min(Ydata)-spanY, max(Ydata)+spanY))
        self.plotwindow.ax.legend(loc=9)
        self.plotwindow.ax.set_xlabel(r'$10^{-5}m^3/mol$-$at$')
        self.plotwindow.ax.set_ylabel(r'$kJ/mol$-$at$')
        self.plotwindow.show()

    def selectionchange(self, i):
        self.ui.progress_2.setValue(0)
        dict_eos = {'Birch-Murnaghan':'BM', 'Rose-Vinet':'RV', 'Mie-Gruneisen':'MG', 'TB-SMA':'TB',
                    'Murnaghan':'MU', 'Poirier-Tarantola':'PT', 'Morse int. potential':'MP', 'EAM int. potential':'EAM'}
        self.eos_str = dict_eos[self.ui.comboBox.itemText(i)]

        self.ipotparamsdialog.args = (None,)
        self.ui.lineEdit_2.setText('-3e5, 1e-5, 7e10, 4')
        if self.eos_str in ['MP', 'EAM']:
            self.ipotparamsdialog.ui.plainTextEdit.setPlainText('')
            for axi in self.ipotparamsdialog.ax:
                for line in axi.lines:
                    line.remove()

                self.ipotparamsdialog.ax[0].figure.canvas.draw()
            self.ipotparamsdialog.molecule = self.molecule
            self.ipotparamsdialog.eos_str = self.eos_str
            self.ipotparamsdialog.ui.OKbutton.setEnabled(False)
            self.ipotparamsdialog.show()

    def get_EvV(self):
        if self.ui.radioButton.isChecked():
            conv = [(1e-30 * 6.02e23), (0.160218e-18 * 6.02214e23)]
        elif self.ui.radioButton_3.isChecked():
            conv =[1, 1]

        txt = self.ui.EvVText.toPlainText().split('\n')
        data_lst = []
        for ti in txt:
            if len(ti)==0:continue
            if ti[0]=='#': continue
            data_lst.append([float(tii) for tii in ti.split()])
        data = np.array(data_lst)
        ncols = len(data.T)
        return (data[:,i]*conv[i] for i in range(ncols))

    def get_EOS_params(self):
        txt = self.ui.lineEdit_2.text()
        if txt=='':
            return -3e5,9e-6,7e10,4
        return  [float(ti) for ti in txt.replace(' ','').split(',')]

    def get_el_params(self):
        txt = self.ui.lineEdit_el.text()
        if txt=='':
            return 3e-01, -1e+04, 5e-04, 1e-06
        return  [float(ti) for ti in txt.replace(' ','').split(',')]

    def get_C(self):
        txt = self.ui.elastic_constants.toPlainText().replace('XX',' ').replace('YY',' ').replace('ZZ',' ').replace('XY',' ').replace('YZ',' ').replace('ZX',' ').split('\n')
        data_lst = []
        for ti in txt:
            if len(ti)==0:continue
            if ti[0]=='#': continue
            data_lst.append([float(tii) for tii in ti.replace('\t',' ').split()])
        data = np.array(data_lst)

        return data

    def on_check_el(self):
        self.state_el = self.check_el.isChecked()
        self.ui.lineEdit_el.setEnabled(self.state_el)

    def on_check_def(self):
        self.state_def = self.check_def.isChecked()
        self.ui.lineEdit_def.setEnabled(self.state_def)

    def on_check_anh(self):
        self.state_anh = self.check_anh.isChecked()
        self.ui.lineEdit_anh.setEnabled(self.state_anh)

    def on_check_xs(self):
        self.state_xs = self.check_xs.isChecked()
        self.ui.lineEdit_xs.setEnabled(self.state_xs)

    def on_pushButton_fitEOS(self):
        self.ui.progress_2.setValue(33)
        self.eos = getattr(dt_potentials, self.eos_str)(*self.ipotparamsdialog.args)

        Vdata, Edata = self.get_EvV()
        self.Vdata = Vdata
        self.Edata = Edata

        initial_guess = self.get_EOS_params()
        self.eos.fitEOS(Vdata, Edata, initial_parameters = initial_guess, fit=True)

        self.ui.lineEdit_2.setText(', '.join(['%.9e' % (p) for p in self.eos.pEOS]))
        self.ui.progress_2.setValue(100)

    def on_pushButton_calc_nu(self):
        C = self.get_C()
        BR, BV, B, GR, GV, S, AU, nu = dt_poisson_ratio(C, quiet=True)
        self.ui.lineEdit_3.setText('%.3f'%(nu))
        self.ui.lineEdit_4.setText('%.1f'%(BR*10))
        self.ui.lineEdit_5.setText('%.1f'%(BV*10))
        self.ui.lineEdit_6.setText('%.1f'%(B*10))
        self.ui.lineEdit_9.setText('%.1f'%(GR*10))
        self.ui.lineEdit_8.setText('%.1f'%(GV*10))
        self.ui.lineEdit_7.setText('%.1f'%(S*10))
        self.ui.lineEdit_10.setText('%.2f'%(AU))

    def on_pushButton_Cp(self):
        self.molecule.nu = float(self.ui.lineEdit_3.text())
        self.molecule.mass = float(self.ui.lineEdit.text())

        self.molecule.p_anh = [float(si) for si in self.ui.lineEdit_anh.text().replace(',',' ').split()] if self.state_anh else [0,1]
        self.molecule.p_el = [float(si) for si in self.ui.lineEdit_el.text().replace(',',' ').split()] if self.state_el else [0,0,0,0]
        self.molecule.p_def = [float(si) for si in self.ui.lineEdit_def.text().replace(',',' ').split()] if self.state_def else [100, 1, 1000, 0.1]
        self.molecule.p_xs = [float(si) for si in self.ui.lineEdit_xs.text().replace(',',' ').split()] if self.state_xs else [0,0,0]

        self.molecule.initial_params = [float(si) for si in self.ui.lineEdit_2.text().replace(',',' ').split()]

        if self.eos_str in ['MP','EAM']:
            args = [self.molecule.formula, self.molecule.cell, self.molecule.basis, self.molecule.cutoff, self.molecule.number_of_NNs]
        else:
            args = [None]
        self.molecule.set_eos(self.eos_str, args)


        self.cp_window.debye_run(self.molecule, self.ui.progress)
        self.cp_window.show()

    def on_pushButton_loaddata(self):
        self.doscardialog.show()

    def on_pushButton_loadSUMMARY(self):
        self.summarydialog.mass = self.ui.lineEdit
        self.summarydialog.EvVtext = self.ui.EvVText
        self.summarydialog.show()

    def on_pushButton_loadOUTCAR(self):
        self.outcardialog.elastic_constants = self.ui.elastic_constants
        self.outcardialog.show()

    def on_pushButton_calculate_el(self):
        Vdata, Edata = self.get_EvV()
        self.Vdata = Vdata
        try:
            E, N, Ef = dt_load_doscar(self.doscardialog.filepath+'/DOSCAR.', list_filetags = [i+1 for i in range(len(Vdata))])
            p_el_initial = self.get_el_params()
            p_electronic = dt_fit_electronic(self.Vdata, p_el_initial,E,N,Ef)
            self.ui.lineEdit_el.setText(', '.join(['%.9e' % (p) for p in p_electronic]))
        except Exception as e:
            print(e)
            self.warningdialog.ui.label.setText(str(e)+"\n\nProbably you haven't set a filepath to the eDOS data yet.")
            self.warningdialog.show()


# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'heatcapacitywindow.ui'
##
## Created by: Qt User Interface Compiler version 6.4.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QHBoxLayout, QHeaderView,
    QLabel, QLineEdit, QMainWindow, QMenuBar,
    QProgressBar, QPushButton, QRadioButton, QSizePolicy,
    QStatusBar, QTableWidget, QTableWidgetItem, QWidget)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(818, 578)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.label_3 = QLabel(self.centralwidget)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(720, 50, 51, 20))
        self.lineEdit_2 = QLineEdit(self.centralwidget)
        self.lineEdit_2.setObjectName(u"lineEdit_2")
        self.lineEdit_2.setGeometry(QRect(600, 50, 113, 24))
        self.label = QLabel(self.centralwidget)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(490, 20, 101, 21))
        self.label_2 = QLabel(self.centralwidget)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(490, 50, 101, 21))
        self.label_9 = QLabel(self.centralwidget)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setGeometry(QRect(600, 170, 21, 21))
        self.tableWidget = QTableWidget(self.centralwidget)
        if (self.tableWidget.columnCount() < 2):
            self.tableWidget.setColumnCount(2)
        __qtablewidgetitem = QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(0, __qtablewidgetitem)
        __qtablewidgetitem1 = QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(1, __qtablewidgetitem1)
        if (self.tableWidget.rowCount() < 8):
            self.tableWidget.setRowCount(8)
        __qtablewidgetitem2 = QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(0, __qtablewidgetitem2)
        __qtablewidgetitem3 = QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(1, __qtablewidgetitem3)
        __qtablewidgetitem4 = QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(2, __qtablewidgetitem4)
        __qtablewidgetitem5 = QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(3, __qtablewidgetitem5)
        __qtablewidgetitem6 = QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(4, __qtablewidgetitem6)
        __qtablewidgetitem7 = QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(5, __qtablewidgetitem7)
        __qtablewidgetitem8 = QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(6, __qtablewidgetitem8)
        __qtablewidgetitem9 = QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(7, __qtablewidgetitem9)
        __qtablewidgetitem10 = QTableWidgetItem()
        __qtablewidgetitem10.setFlags(Qt.ItemIsSelectable|Qt.ItemIsDragEnabled|Qt.ItemIsDropEnabled|Qt.ItemIsUserCheckable|Qt.ItemIsEnabled);
        self.tableWidget.setItem(0, 1, __qtablewidgetitem10)
        __qtablewidgetitem11 = QTableWidgetItem()
        __qtablewidgetitem11.setFlags(Qt.ItemIsSelectable|Qt.ItemIsDragEnabled|Qt.ItemIsDropEnabled|Qt.ItemIsUserCheckable|Qt.ItemIsEnabled);
        self.tableWidget.setItem(1, 1, __qtablewidgetitem11)
        __qtablewidgetitem12 = QTableWidgetItem()
        __qtablewidgetitem12.setFlags(Qt.ItemIsSelectable|Qt.ItemIsDragEnabled|Qt.ItemIsDropEnabled|Qt.ItemIsUserCheckable|Qt.ItemIsEnabled);
        self.tableWidget.setItem(2, 1, __qtablewidgetitem12)
        __qtablewidgetitem13 = QTableWidgetItem()
        __qtablewidgetitem13.setFlags(Qt.ItemIsSelectable|Qt.ItemIsDragEnabled|Qt.ItemIsDropEnabled|Qt.ItemIsUserCheckable|Qt.ItemIsEnabled);
        self.tableWidget.setItem(3, 1, __qtablewidgetitem13)
        __qtablewidgetitem14 = QTableWidgetItem()
        __qtablewidgetitem14.setFlags(Qt.ItemIsSelectable|Qt.ItemIsDragEnabled|Qt.ItemIsDropEnabled|Qt.ItemIsUserCheckable|Qt.ItemIsEnabled);
        self.tableWidget.setItem(4, 1, __qtablewidgetitem14)
        __qtablewidgetitem15 = QTableWidgetItem()
        __qtablewidgetitem15.setFlags(Qt.ItemIsSelectable|Qt.ItemIsDragEnabled|Qt.ItemIsDropEnabled|Qt.ItemIsUserCheckable|Qt.ItemIsEnabled);
        self.tableWidget.setItem(5, 1, __qtablewidgetitem15)
        __qtablewidgetitem16 = QTableWidgetItem()
        __qtablewidgetitem16.setFlags(Qt.ItemIsSelectable|Qt.ItemIsDragEnabled|Qt.ItemIsDropEnabled|Qt.ItemIsUserCheckable|Qt.ItemIsEnabled);
        self.tableWidget.setItem(6, 1, __qtablewidgetitem16)
        __qtablewidgetitem17 = QTableWidgetItem()
        __qtablewidgetitem17.setFlags(Qt.ItemIsSelectable|Qt.ItemIsDragEnabled|Qt.ItemIsDropEnabled|Qt.ItemIsUserCheckable|Qt.ItemIsEnabled);
        self.tableWidget.setItem(7, 1, __qtablewidgetitem17)
        self.tableWidget.setObjectName(u"tableWidget")
        self.tableWidget.setGeometry(QRect(500, 200, 311, 271))
        self.label_5 = QLabel(self.centralwidget)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(490, 170, 41, 21))
        self.label_4 = QLabel(self.centralwidget)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(720, 20, 51, 20))
        self.lineEdit_3 = QLineEdit(self.centralwidget)
        self.lineEdit_3.setObjectName(u"lineEdit_3")
        self.lineEdit_3.setGeometry(QRect(530, 170, 41, 24))
        self.horizontalLayoutWidget = QWidget(self.centralwidget)
        self.horizontalLayoutWidget.setObjectName(u"horizontalLayoutWidget")
        self.horizontalLayoutWidget.setGeometry(QRect(10, 10, 471, 421))
        self.horizontalLayout = QHBoxLayout(self.horizontalLayoutWidget)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.label_8 = QLabel(self.centralwidget)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setGeometry(QRect(670, 170, 51, 21))
        self.lineEdit = QLineEdit(self.centralwidget)
        self.lineEdit.setObjectName(u"lineEdit")
        self.lineEdit.setGeometry(QRect(600, 20, 113, 24))
        self.label_6 = QLabel(self.centralwidget)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setGeometry(QRect(580, 170, 16, 21))
        self.pushButton = QPushButton(self.centralwidget)
        self.pushButton.setObjectName(u"pushButton")
        self.pushButton.setEnabled(True)
        self.pushButton.setGeometry(QRect(700, 480, 80, 24))
        self.lineEdit_4 = QLineEdit(self.centralwidget)
        self.lineEdit_4.setObjectName(u"lineEdit_4")
        self.lineEdit_4.setGeometry(QRect(620, 170, 41, 24))
        self.label_7 = QLabel(self.centralwidget)
        self.label_7.setObjectName(u"label_7")
        self.label_7.setGeometry(QRect(490, 150, 121, 16))
        self.comboBox = QComboBox(self.centralwidget)
        self.comboBox.setObjectName(u"comboBox")
        self.comboBox.setGeometry(QRect(720, 170, 61, 24))
        self.label_10 = QLabel(self.centralwidget)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setGeometry(QRect(700, 170, 21, 21))
        self.label_11 = QLabel(self.centralwidget)
        self.label_11.setObjectName(u"label_11")
        self.label_11.setGeometry(QRect(790, 170, 21, 21))
        self.comboBox_2 = QComboBox(self.centralwidget)
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.setObjectName(u"comboBox_2")
        self.comboBox_2.setGeometry(QRect(350, 440, 131, 24))
        self.label_12 = QLabel(self.centralwidget)
        self.label_12.setObjectName(u"label_12")
        self.label_12.setGeometry(QRect(258, 440, 91, 21))
        self.progress = QProgressBar(self.centralwidget)
        self.progress.setObjectName(u"progress")
        self.progress.setGeometry(QRect(700, 500, 81, 16))
        self.progress.setValue(0)
        self.progress.setTextVisible(False)
        self.progress.setInvertedAppearance(False)
        self.radioButton = QRadioButton(self.centralwidget)
        self.radioButton.setObjectName(u"radioButton")
        self.radioButton.setGeometry(QRect(490, 90, 91, 22))
        self.radioButton.setChecked(True)
        self.radioButton_2 = QRadioButton(self.centralwidget)
        self.radioButton_2.setObjectName(u"radioButton_2")
        self.radioButton_2.setGeometry(QRect(550, 90, 141, 22))
        self.radioButton_3 = QRadioButton(self.centralwidget)
        self.radioButton_3.setObjectName(u"radioButton_3")
        self.radioButton_3.setGeometry(QRect(690, 90, 121, 22))
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 818, 21))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"debyetools GUI (lite) - Heat Capacity", None))
        self.label_3.setText(QCoreApplication.translate("MainWindow", u"GPa", None))
        self.lineEdit_2.setText(QCoreApplication.translate("MainWindow", u"0 30 10", None))
        self.label.setText(QCoreApplication.translate("MainWindow", u"Temperature range:", None))
        self.label_2.setText(QCoreApplication.translate("MainWindow", u"Pressure:", None))
        self.label_9.setText(QCoreApplication.translate("MainWindow", u"to:", None))
        ___qtablewidgetitem = self.tableWidget.horizontalHeaderItem(0)
        ___qtablewidgetitem.setText(QCoreApplication.translate("MainWindow", u"Values", None));
        ___qtablewidgetitem1 = self.tableWidget.verticalHeaderItem(0)
        ___qtablewidgetitem1.setText(QCoreApplication.translate("MainWindow", u"G+TS (T=298.15)", None));
        ___qtablewidgetitem2 = self.tableWidget.verticalHeaderItem(1)
        ___qtablewidgetitem2.setText(QCoreApplication.translate("MainWindow", u"S (T=298.15)", None));
        ___qtablewidgetitem3 = self.tableWidget.verticalHeaderItem(2)
        ___qtablewidgetitem3.setText(QCoreApplication.translate("MainWindow", u"C0", None));
        ___qtablewidgetitem4 = self.tableWidget.verticalHeaderItem(3)
        ___qtablewidgetitem4.setText(QCoreApplication.translate("MainWindow", u"C1", None));
        ___qtablewidgetitem5 = self.tableWidget.verticalHeaderItem(4)
        ___qtablewidgetitem5.setText(QCoreApplication.translate("MainWindow", u"C2", None));
        ___qtablewidgetitem6 = self.tableWidget.verticalHeaderItem(5)
        ___qtablewidgetitem6.setText(QCoreApplication.translate("MainWindow", u"C3", None));
        ___qtablewidgetitem7 = self.tableWidget.verticalHeaderItem(6)
        ___qtablewidgetitem7.setText(QCoreApplication.translate("MainWindow", u"C4", None));
        ___qtablewidgetitem8 = self.tableWidget.verticalHeaderItem(7)
        ___qtablewidgetitem8.setText(QCoreApplication.translate("MainWindow", u"C5", None));

        __sortingEnabled = self.tableWidget.isSortingEnabled()
        self.tableWidget.setSortingEnabled(False)
        ___qtablewidgetitem9 = self.tableWidget.item(0, 1)
        ___qtablewidgetitem9.setText(QCoreApplication.translate("MainWindow", u"J/mol-at", None));
        ___qtablewidgetitem10 = self.tableWidget.item(1, 1)
        ___qtablewidgetitem10.setText(QCoreApplication.translate("MainWindow", u"J/K/mol-at", None));
        ___qtablewidgetitem11 = self.tableWidget.item(2, 1)
        ___qtablewidgetitem11.setText(QCoreApplication.translate("MainWindow", u"T^(0)", None));
        ___qtablewidgetitem12 = self.tableWidget.item(3, 1)
        ___qtablewidgetitem12.setText(QCoreApplication.translate("MainWindow", u"T^(1)", None));
        ___qtablewidgetitem13 = self.tableWidget.item(4, 1)
        ___qtablewidgetitem13.setText(QCoreApplication.translate("MainWindow", u"T^(-2)", None));
        ___qtablewidgetitem14 = self.tableWidget.item(5, 1)
        ___qtablewidgetitem14.setText(QCoreApplication.translate("MainWindow", u"T^(2)", None));
        ___qtablewidgetitem15 = self.tableWidget.item(6, 1)
        ___qtablewidgetitem15.setText(QCoreApplication.translate("MainWindow", u"T^(-0.5)", None));
        ___qtablewidgetitem16 = self.tableWidget.item(7, 1)
        ___qtablewidgetitem16.setText(QCoreApplication.translate("MainWindow", u"T^(-3)", None));
        self.tableWidget.setSortingEnabled(__sortingEnabled)

        self.label_5.setText(QCoreApplication.translate("MainWindow", u"From:", None))
        self.label_4.setText(QCoreApplication.translate("MainWindow", u"K", None))
        self.lineEdit_3.setText(QCoreApplication.translate("MainWindow", u"298.15", None))
        self.label_8.setText(QCoreApplication.translate("MainWindow", u"K", None))
        self.lineEdit.setText(QCoreApplication.translate("MainWindow", u"0.1 1000.1 100", None))
        self.label_6.setText(QCoreApplication.translate("MainWindow", u"K", None))
        self.pushButton.setText(QCoreApplication.translate("MainWindow", u"re-calculate", None))
        self.lineEdit_4.setText(QCoreApplication.translate("MainWindow", u"1000.1", None))
        self.label_7.setText(QCoreApplication.translate("MainWindow", u"FactSage parameters:", None))
        self.label_10.setText(QCoreApplication.translate("MainWindow", u"P:", None))
        self.label_11.setText(QCoreApplication.translate("MainWindow", u"GPa", None))
        self.comboBox_2.setItemText(0, QCoreApplication.translate("MainWindow", u"New Item", None))
        self.comboBox_2.setItemText(1, QCoreApplication.translate("MainWindow", u"New Item", None))
        self.comboBox_2.setItemText(2, QCoreApplication.translate("MainWindow", u"New Item", None))
        self.comboBox_2.setItemText(3, QCoreApplication.translate("MainWindow", u"New Item", None))
        self.comboBox_2.setItemText(4, QCoreApplication.translate("MainWindow", u"New Item", None))
        self.comboBox_2.setItemText(5, QCoreApplication.translate("MainWindow", u"New Item", None))
        self.comboBox_2.setItemText(6, QCoreApplication.translate("MainWindow", u"New Item", None))
        self.comboBox_2.setItemText(7, QCoreApplication.translate("MainWindow", u"New Item", None))
        self.comboBox_2.setItemText(8, QCoreApplication.translate("MainWindow", u"New Item", None))
        self.comboBox_2.setItemText(9, QCoreApplication.translate("MainWindow", u"New Item", None))
        self.comboBox_2.setItemText(10, QCoreApplication.translate("MainWindow", u"New Item", None))

        self.label_12.setText(QCoreApplication.translate("MainWindow", u"property to plot:", None))
        self.radioButton.setText(QCoreApplication.translate("MainWindow", u"Slater", None))
        self.radioButton_2.setText(QCoreApplication.translate("MainWindow", u"Dugdale-MacDonald", None))
        self.radioButton_3.setText(QCoreApplication.translate("MainWindow", u"mean-free-volume", None))
    # retranslateUi




import numpy as np
from scipy.optimize import leastsq

NAv  = 0.6022140857e24
kB   = 0.138064852e-22

class Electronic:
    """
    Implementation of the electronic contribution to the free energy.

    :param float params: N(Ef)(V) function parameters.
    """
    def __init__(self, *params: np.ndarray):
        self.pel = params
        self.r=1
        for n,q in enumerate(params):
            setattr(self,'q'+str(n),q)

    def NfV(self, V: float) -> float:
        """
        N(Ef)(V)

        :param float V: Volume.
        :return: N(Ef)(V)
        :rtype: float
        """
        return self.q0*V**0 + self.q1*V**1 + self.q2*V**2 + self.q3*V**3

    def dNfVdV_T(self, V: float) -> float:
        """
        derivative of N(Ef)(V)

        :param V: Volume.
        :type V: float
        :return: derivative of N(Ef)(V)
        :rtype: float
        """
        return 3*V**2*self.q3+2*V*self.q2+self.q1

    def d2NfVdV2_T(self, V: float) -> float:
        """
        derivative of N(Ef)(V)

        :param V: Volume.
        :type V: float
        :return: derivative of N(Ef)(V)
        :rtype: float
        """
        return 6*V*self.q3+2*self.q2

    def d3NfVdV3_T(self, V: float) -> float:
        """
        derivative of N(Ef)(V)

        :param V: Volume.
        :type V: float
        :return: derivative of N(Ef)(V)
        :rtype: float
        """
        return 6*self.q3
    def d4NfVdV4_T(self, V: float) -> float:
        """
        derivative of N(Ef)(V)

        :param V: Volume.
        :type V: float
        :return: derivative of N(Ef)(V)
        :rtype: float
        """
        return 0

    def E(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Electronic energy

        :param T: Temperature.
        :type T: float|np.ndarray
        :param V: Volume.
        :type V: float|np.ndarray
        :return: E_el
        :rtype: float|np.ndarray
        """
        return (1/6)*np.pi**2*NAv*self.r*kB**2*T**2*self.NfV(V)/(0.160218e-18)

    def S(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Electronic entropy.

        :param T: Temperature.
        :type T: float|np.ndarray
        :param V: Volume.
        :type V: float|np.ndarray
        :return: S_el
        :rtype: float|np.ndarray
        """
        return (2/6)*np.pi**2*NAv*self.r*kB**2*T*self.NfV(V)/(0.160218e-18)
    def F(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return - (1/6)*np.pi**2*NAv*self.r*kB**2*T**2*self.NfV(V)/(0.160218e-18)

    def dFdV_T(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return - (1/6)*np.pi**2*NAv*self.r*kB**2*T**2*self.dNfVdV_T(V)/(0.160218e-18)
    def dFdT_V(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return - 2*T*np.pi**2*NAv*self.r*kB**2*self.NfV(V)*(1/6)/(0.160218e-18)
    def d2FdT2_V(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return - 2*np.pi**2*NAv*self.r*kB**2*self.NfV(V)*(1/6)/(0.160218e-18)
    def d2FdV2_T(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return - (1/6)*np.pi**2*NAv*self.r*kB**2*T**2*self.d2NfVdV2_T(V)/(0.160218e-18)
    def d3FdV3_T(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return - (1/6)*np.pi**2*NAv*self.r*kB**2*T**2*self.d3NfVdV3_T(V)/(0.160218e-18)
    def d4FdV4_T(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return - (1/6)*np.pi**2*NAv*self.r*kB**2*T**2*self.d4NfVdV4_T(V)/(0.160218e-18)

    def d2FdVdT(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return -2*np.pi**2*NAv*self.r*kB**2*T*self.dNfVdV_T(V)*(1/6)/(0.160218e-18)
    def d3FdV2dT(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return -2*np.pi**2*NAv*self.r*kB**2*T*self.d2NfVdV2_T(V)*(1/6)/(0.160218e-18)
    def d3FdVdT2(self, T: float|np.ndarray, V: float|np.ndarray) -> float|np.ndarray:
        """
        Derivative of the electronic contribution to the free energy.

        :param float|np.ndarray T: Temperature.
        :param float|np.ndarray V: Volume.
        :return: F_el
        :rtype: float|np.ndarray
        """
        return -2*np.pi**2*NAv*self.r*kB**2*self.dNfVdV_T(V)*(1/6)/(0.160218e-18)

def fit_electronic(Vs: np.ndarray, p_el: np.ndarray, E: np.ndarray, N: np.ndarray, Ef: np.ndarray, ixss: int = 8, ixse: int = -1) -> np.ndarray:
    """
    Fitting procedure for the N(Ef)(V) function.

    :param np.ndarray Vs: Volumes.
    :param np.ndarray p_el: Initial parameters.
    :param np.ndarray E: Matrix with energies at each level for each volume.
    :param np.ndarray N: Matrix with densities of state at each level for each volume.
    :param np.ndarray Ef: Fermi levels as function of temperature.
    :param int ixse: (optional) eDOS subset index.
    :param intixss: (optional) eDOS subset index.

    :retun np.ndarray: optimized parameters.
    """

    V = np.array(Vs)
    ix_V0=4
    EfV0 = float(Ef[ix_V0])
    ixs=[i for i,x in enumerate(E[ix_V0]) if x>=Ef[ix_V0]]
    E1 = float(E[ix_V0][ixs[0]-1])
    E2 = float(E[ix_V0][ixs[0]])
    N1 = float(N[ix_V0][ixs[0]-1])
    N2 = float(N[ix_V0][ixs[0]])
    NfV0 = (EfV0 - E1)*(N2 - N1)/(E2 - E1) + N1
    NfV = np.array([NfV0*np.sqrt(ef/EfV0) for ef in Ef][ixss:ixse])
    P2 = leastsq(NfV2m, p_el, args=(V[ixss:ixse], NfV),maxfev=1000)

    P2 = P2[0]

    return P2

def NfV_poly_fun(V: float, _A: float, _B: float, _C: float, _D: float) -> float:
    """
    Polynomial model for N(Ef)(V) for min.

    :param V: Volume/
    :type V: float
    :param _A: param.
    :type _A: float
    :param _B: param.
    :type _B: float
    :param _C: param.
    :type _C: float
    :param _D: param.
    :type _D: float
    :return: polynimial for minimization.
    :rtype: float
    """
    return _A + _B*V + _C*V**2 + _D*V**3

def NfV2m(P: np.ndarray, Vdata: np.ndarray, NfVdata: np.ndarray) -> np.ndarray:
    """
    Error function for minimizaiton.

    :param P: Parameters.
    :type P: np.ndarray
    :param Vdata: Volume data.
    :type Vdata: np.ndarray
    :param NfVdata: N(Ef)(V) data.
    :type NfVdata: np.ndarray
    :return: err.
    :rtype: np.ndarray
    """
    NfVcalc = [NfV_poly_fun(Vi, P[0], P[1], P[2], P[3]) for Vi in Vdata]
    return (NfVcalc-NfVdata)**2



import numpy as np
import itertools as it
import debyetools.aux_functions as afn
from typing import Tuple

def neighbor_list(size: np.ndarray, basis: np.ndarray, cell: np.ndarray, cutoff: float)->Tuple[np.ndarray,np.ndarray,np.ndarray,np.ndarray,np.ndarray]:
    """ calculate a list i, j, dij where i and j are a pair of atoms of
    indexes i and j, respectively, and dij is the distance between them.

    :param np.ndarray size: Number of times we are replicating the primitive cel
    :param np.ndarray basis: atoms position within a single primitive cell
    :param np.ndarray cell: the primitive cell
    :param float cutoff: cut-off distance
    :return: D, I , J
    :rtype: Tuple[np.ndarray, np.ndarray, np.ndarray]
    
    """
    max_depth = np.array([2*int(m) for m in cutoff/np.linalg.norm(cell, axis=1)])
    center = np.array([0,0,0])
    basis = np.dot(basis, cell)
    size_g = size + 2*max_depth

    cell_coords_centered = afn.generate_cells_coordinates(size, cell, center)
    cell_coords_centered_g = afn.generate_cells_coordinates(size_g, cell, center-max_depth)

    XCs = []
    Is = []
    Js = []
    CIXs = []


    ixs = np.arange(len(basis))
    jxs = np.arange(len(basis))

    for cell_coords_i in cell_coords_centered:
        ix_neighbor = np.where(np.all(abs(cell_coords_centered_g-cell_coords_i)<=cutoff, axis=1))[0]
        Xs = np.array([cell_coords_i,]*len(basis))+basis

        for ixx, cell_coords_i_g in enumerate(cell_coords_centered_g[ix_neighbor]):
            Xsg = np.array([cell_coords_i_g,]*len(basis))+basis
            for x, xg in it.product(Xs, Xsg):
                XCs.append((x-xg)**2)
            for i, j in it.product(ixs, jxs):
                Is.append(i)
                Js.append(j)
                CIXs.append(ix_neighbor[ixx])


    XCs = np.array(XCs)
    Is = np.array(Is)
    Js = np.array(Js)
    CIXs = np.array(CIXs)

    CX = np.sum(XCs/cutoff**2, axis=1)

    ix = np.where(np.all([CX<=1, CX>0], axis=0))[0]
    distances = np.sqrt(np.sum(XCs[ix], axis=1))

    return distances, Is[ix], Js[ix], cell_coords_centered_g, CIXs[ix]

def pair_analysis(atom_types, cutoff, basis, cell, prec=10, full=False):
    """
    run a pair analysis of a crystal structure of almost any type of symmetry.

    :param str atom_types: the types of each atom in the primitive cell in the same order as the basis vectors.
    :param float cutoff: cut-off distance
    :param np.ndarray basis: atoms position within a single primitive cell
    :param np.ndarray cell: the primitive cell
    :param int prec: precision.
    :param boolean full: if True, returns also data of ghost cells.
    :return:  pair distance, pair number, pair types
    :rtype: Tuple[np.ndarray,np.ndarray,np.ndarray]
    """
    size=np.array([1,1,1])
    dAxBy, iAxBy, jAxBy, cells_cohordniates, ij_ccix  = neighbor_list(size, basis, cell, cutoff)
    if cutoff is not None:
        maxdjx = np.where(dAxBy <= cutoff)
        dAxBy, iAxBy, jAxBy = dAxBy[maxdjx], iAxBy[maxdjx], jAxBy[maxdjx]
    dAxBy = np.array([np.round(d,prec) for d in dAxBy])
    res_2 = dAxBy, iAxBy, jAxBy
    nat = np.prod(size)*len(basis)

    combs_types,types_all = afn.c_types(atom_types)

    bins_dAxBy =list(set([li for li in list(set(np.append(dAxBy, [cutoff])))]))
    bins_dAxBy.sort()
    distances = bins_dAxBy[:-1]

    ptlst = []
    pairtype = 0
    for i,j,d in zip(iAxBy,jAxBy,dAxBy):
        for ii in range(len(combs_types)):
            if (types_all[i]+'-'+types_all[j] == combs_types[ii]) or (types_all[j]+'-'+types_all[i] == combs_types[ii]):
                pairtype = ii
        ptlst.append(pairtype)

    ptlst = np.array(ptlst)

    ds = ['' for x in range(len(combs_types))]
    for i in range(len(combs_types)):
        ds[i] = np.array([d for d in dAxBy[np.where(ptlst==i)[0]]])
    hs = ['' for x in range(len(combs_types))]
    bs = ['' for x in range(len(combs_types))]
    for i in range(len(combs_types)):
        hs[i], bs[i] = np.histogram(ds[i], bins=bins_dAxBy)
    tot_num_bonds_per_molecule = np.array(hs).T
    num_bonds_per_formula = tot_num_bonds_per_molecule/nat

    if full:
        return np.array(distances), num_bonds_per_formula, combs_types, res_2[0], res_2[1], res_2[2], cells_cohordniates, ij_ccix
    else:
        return np.array(distances), num_bonds_per_formula, combs_types



import re
import uuid

mac_lst = re.findall('..', '%012x' % uuid.getnode())
key_lst = []
A = 'abcdEF123456gHIJklMNop556372834q6rs6TU7vWxyzAbcDEfgHiJklMNoPQrsTUVwxYZabcdEFgHIJklMNopqrsTUvWxyzAbcDEfgHiJklMNoPQrsTUVwxYZ'
B = 'Abc123DEf5g6H7i8J8k9l0MNoPQrsTUVwxYZAbc7D6E5f4g3H2i2J2klMNoPQrsTUVwxYZabcdEFgHIJklMNopqrsTUvWxyzabcdEFgHIJklMNopqrsTUvWxyz'
for l in mac_lst:
    # print(l.isnumeric())
    if  l.isnumeric() is not True:
        k = ''.join(format(ord(i), '08b') for i in l)
    else:
        a=int(l[0])
        b=int(l[1])
        c= str(a*b)
        k = ''.join(format(ord(i), '08b') for i in c)
    for i, a in enumerate(k):
        if int(a) == 0:
            key_lst.append(A[i])
        if int(a) == 1:
            key_lst.append(B[i])
key = ''.join(key_lst)

with open('keydtgl', 'w') as f:
    f.write(key)



import numpy as np
np.seterr(divide='ignore')

class Anharmonicity:
    """
    Instance for the excess contribution to the free energy.

    :param float s0,s1,s2: Parameters of the A(V) term.
    """

    def __init__(self, s0: float, s1: float, s2: float) -> None:
        self.panh = s0, s1, s2
        self.s0 = s0
        self.s1 = s1
        self.s2 = s2

    def A(self, V: float) -> float:
        """
        A(V) = s0+s0*V+s1*V**2, where A is the polynomial model for the excess contribution to the free energy, A(V)*T.

        :param float V: Volume
        :return: s0+s0*V+s1*V**2
        :rtype: float
        """
        return self.s0 + self.s1 * V + self.s2 * V ** 2

    def dAdV_T(self, V: float) -> float:
        """
        Volume derivative of A at fixed T.

        :param float V: Volume
        :return: s1+2*V*s2
        :rtype: float
        """
        return 2 * V * self.s2 + self.s1

    def d2AdV2_T(self, V: float) -> float:
        """
        Second order volume derivative of A at fixed T.

        :param float V: Volume
        :return: 2*s2
        :rtype: float
        """
        return 2 * self.s2

    def d3AdV3_T(self, V: float) -> float:
        """
        Third order volume derivative of A at fixed T.

        :param float V: Volume
        :return: 0
        :rtype: float
        """
        return 0

    def d4AdV4_T(self, V: float) -> float:
        """
        Fourth order volume derivative of A at fixed T.

        :param float V: Volume.
        :return: 0
        :rtype: float.
        """
        return 0

    def E(self, T, V: float) -> float:
        """
        Internal energy due the excess term.

        :param float T: Temperature.
        :param float V: Volume.
        :return: A(V)*T**2/2
        :rtype: float
        """
        return 1 / 2 * self.A(V) * T ** 2

    def S(self, T, V: float) -> float:
        """
        Entropy due the excess term.

        :param float T: Temperature.
        :param float V: Volume.
        :return: A(V)*T
        :rtype: float
        """
        return self.A(V) * T

    def F(self, T, V: float) -> float:
        """
        Free energy due the excess term.

        :param float T: Temperature.
        :param float V: Volume.
        :return: -A(V)*T**2/2
        :rtype: float
        """
        return -1 / 2 * self.A(V) * T ** 2

    def dFdV_T(self, T, V: float) -> float:
        """
        Volume derivative of the free energy due the excess term, at fixed T.

        :param float T: Temperature.
        :param float V: Volume.
        :return: -(dA(V)/dV)_T*T**2/2
        :rtype: float
        """
        return -1 / 2 * self.dAdV_T(V) * T ** 2

    def dFdT_V(self, T, V: float) -> float:
        """
        Temperature derivative of the free energy due the excess term, at fixed V.

        :param float T: Temperature.
        :param float V: Volume.
        :return: -A(V)
        :rtype: float
        """
        return -T*self.A(V)

    def d2FdT2_V(self, T, V: float) -> float:
        """
        Second order Temperature derivative of the free energy due the excess term, at fixed V.

        :param float T: Temperature.
        :param float V: Volume.
        :return: -A(V)
        :rtype: float
        """
        return -self.A(V)

    def d2FdV2_T(self, T, V: float) -> float:
        """
        Second order volume derivative of the free energy due the excess term, at fixed T.

        :param float T: Temperature.
        :param float V: Volume.
        :return: -(d2A(V)/dV2)_T*T**2/2
        :rtype: float
        """
        return -self.d2AdV2_T(V) * T ** 2 / 2

    def d3FdV3_T(self, T, V: float) -> float:
        """
        Third order volume derivative of the free energy due the excess term, at fixed T.

        :param float T: Temperature.
        :param float V: Volume.
        :return: -(d3A(V)/dV3)_T*T**2/2
        :rtype: float
        """
        return -self.d3AdV3_T(V) * T ** 2 / 2

    def d4FdV4_T(self, T, V: float) -> float:
        """
        Fourth order volume derivative of the free energy due the excess term, at fixed T.

        :param float T: Temperature.
        :param float V: Volume.
        :return: -(d4A(V)/dV4)_T*T**2/2
        :rtype: float
        """
        return -self.d4AdV4_T(V) * T ** 2 / 2

    def d2FdVdT(self, T, V: float) -> float:
        """
        Second order derivative of the free energy due the excess term, with respect to T and V.

        :param float T: Temperature.
        :param float V: Volume.
        :return: -(dA(V)/dV)_T*T
        :rtype: float
        """
        return -self.dAdV_T(V) * T

    def d3FdV2dT(self, T, V: float) -> float:
        """
        Third order derivative of the free energy due the excess term, with respect to T, V, and V.

        :param float T: Temperature.
        :param float V: Volume.
        :return: -(d2A(V)/dV2)_T*T
        :rtype: float
        """
        return -self.d2AdV2_T(V) * T

    def d3FdVdT2(self, T, V: float) -> float:
        """
        Third order derivative of the free energy due the excess term, with respect to T, T, and V.

        :param float T: Temperature.
        :param float V: Volume.
        :return: -(dA(V)/dV)_T
        :rtype: float
        """
        return -self.dAdV_T(V)


class intAnharmonicity:
    def __init__(self, a0: float = 0, m0: float = 1, V0: float = 1):
        """
        Instantiate the corrections required to consider temperature dependence on the Debye temperature.

        :param float a0: Parameter of the intrinsic anharmonicity correction function.
        :param float m0: Pseudo-Gruneisen parameter of the intrinsic anharmonicity correction function.
        :param float V0: Equilibrium volume.
        """
        self.pintanh = a0, m0, V0
        self.d3AnhdV2dT_val = None
        self.d3AnhdVdT2_val = None
        self.d2AnhdVdT_val = None
        self.d3AnhdV3_T_val = None
        self.d2AnhdV2_T_val = None
        self.dAnhdV_T_val = None
        self.d4AnhdT4_V_val = None
        self.d3AnhdT3_V_val = None
        self.d2AnhdT2_V_val = None
        self.Anh_val = None
        self.an_val = None
        self.dAnhdT_V_val = None
        self.a0 = a0
        self.m0 = m0
        self.V0 = V0

    def an(self, V: float) -> float:
        """
        Intrinsic anharmonicity parameter.

        :param float V: Volume
        :return: a0*(V/V0)^m0
        :rtype: float
        """
        return self.a0 * (V / self.V0) ** self.m0

    def Anh(self, T, V: float) -> float:
        """
        Intrinsic anharmonicity correction to the Debye temperature.

        :param float T: Temperature.
        :param float V: Volume.
        :returns: Intrinsic anharmonicity correction
        :rtype: float
        """
        self.an_val = self.an(V)
        self.Anh_val = np.exp(1 / 2 * self.an_val * T)
        return self.Anh_val

    def dAnhdT_V(self) -> float:
        """
        Temperature derivative of the intrinsic anharmonicity correction of the Debye temperature, at fixed V.

        :param float V: Volume
        :return: an_val*Anh_val/2
        :rtype: float
        """
        self.dAnhdT_V_val = (1 / 2) * self.an_val * self.Anh_val
        return self.dAnhdT_V_val

    def d2AnhdT2_V(self) -> float:
        """
        Second order temperature derivative of the intrinsic anharmonicity correction of the Debye temperature,
        at fixed V.

        :param float V: Volume
        :return: an_val*(d2Anh_val/dT2)_V/2
        :rtype: float
        """
        self.d2AnhdT2_V_val = (1 / 2) * self.an_val * self.dAnhdT_V_val
        return self.d2AnhdT2_V_val

    def d3AnhdT3_V(self) -> float:
        """
        Third order temperature derivative of the intrinsic anharmonicity correction of the Debye temperature,
        at fixed V.

        :param float V: Volume
        :return: an_val*(d2Anh_val/dT2)_V/2
        :rtype: float
        """
        self.d3AnhdT3_V_val = (1 / 2) * self.an_val * self.d2AnhdT2_V_val
        return self.d3AnhdT3_V_val

    def d4AnhdT4_V(self) -> float:
        """
        Fourth order temperature derivative of the intrinsic anharmonicity correction of the Debye temperature,
        at fixed V.

        :param float V: Volume.
        :return: an_val*(d3Anh_val/dT3)_V/2
        :rtype: float
        """
        self.d4AnhdT4_V_val = (1 / 2) * self.an_val * self.d3AnhdT3_V_val
        return self.d4AnhdT4_V_val

    def dAnhdV_T(self, T: float, V: float) -> float:
        """
        Volume derivative of the intrinsic anharmonicity correction of the Debye temperature, at fixed T.

        :param float T: Temperature
        :param float V: Volume
        :return: m0*T*an_val*Anh_val/(2*V)
        :rtype: float
        """
        self.dAnhdV_T_val = self.m0 * T * self.an_val * self.Anh_val / (2 * V)
        return self.dAnhdV_T_val

    def d2AnhdV2_T(self, T: float, V: float) -> float:
        """
        Second order Volume derivative of the intrinsic anharmonicity correction of the Debye temperature, at fixed T.

        :param float T: Temperature
        :param float V: Volume
        :return: dAnhdV_T_val*(T*an_val*m0+2*m0-2)/(2*V)
        :rtype: float
        """

        self.d2AnhdV2_T_val = self.dAnhdV_T_val * (T * self.an_val * self.m0 + 2 * self.m0 - 2) / (2 * V)
        return self.d2AnhdV2_T_val

    def d3AnhdV3_T(self, T: float, V: float) -> float:
        """
        Third order Volume derivative of the intrinsic anharmonicity correction of the Debye temperature, at fixed T.

        :param float T: Temperature
        :param float V: Volume
        :return: dAnhdV_T_val*(T^2*an_val^2*m0^2+6*T*an_val*m0^2-6*T*an_val*m0+4*m0^2-12*m0+8)/(4*V^2)
        :rtype: float
        """
        self.d3AnhdV3_T_val = self.dAnhdV_T_val * (
                    T ** 2 * self.an_val ** 2 * self.m0 ** 2 + 6 * T * self.an_val * self.m0 ** 2 - 6 * T * self.an_val * self.m0 + 4 * self.m0 ** 2 - 12 * self.m0 + 8) / (
                                          4 * V ** 2)
        return self.d3AnhdV3_T_val

    def d4AnhdV4_T(self, T: float, V: float) -> float:
        """
        Fourth order Volume derivative of the intrinsic anharmonicity correction of the Debye temperature, at fixed T.

        :param float T: Temperature
        :param float V: Volume
        :return: dAnhdV_T_val*(T^3*an_val^3*m0^3+12*T^2*an_val^2*m0^3 - 12*T^2*an_val^2*m0^2+28*T*an_val*m0^3-72*T*an_val*m0^2+44T*an_val*m0+8+m0^3-48*m0^2+88*m0-48)/(8*V^3)
        :rtype: float
        """
        self.d4AnhdT4_V_val = self.dAnhdV_T_val * (
                    T ** 3 * self.an_val ** 3 * self.m0 ** 3 + 12 * T ** 2 * self.an_val ** 2 * self.m0 ** 3 - 12 * T ** 2 * self.an_val ** 2 * self.m0 ** 2 + 28 * T * self.an_val * self.m0 ** 3 - 72 * T * self.an_val * self.m0 ** 2 + 44 * T * self.an_val * self.m0 + 8 * self.m0 ** 3 - 48 * self.m0 ** 2 + 88 * self.m0 - 48) / (
                                          8 * V ** 3)
        return self.d4AnhdT4_V_val

    def d2AnhdVdT(self, T: float) -> float:
        """
        Second order derivative of the intrinsic anharmonicity correction of the Debye temperature,
        with respect to T and V.

        :param float T: Temperature
        :return: dAnhdV_T_val * (1 / T + an_val / 2)
        :rtype: float
        """
        self.d2AnhdVdT_val = self.dAnhdV_T_val * (1 / T + self.an_val / 2)
        return self.d2AnhdVdT_val

    def d3AnhdVdT2(self, T: float) -> float:
        """
        Third order derivative of the intrinsic anharmonicity correction of the Debye temperature,
        with respect to T, T, and V.

        :param float T: Temperature
        :return: dAnhdV_T_val * an_val * (1 / T + an_val / 4)
        :rtype: float
        """

        self.d3AnhdVdT2_val = self.dAnhdV_T_val * self.an_val * (1 / T + self.an_val / 4)
        return self.d3AnhdVdT2_val

    def d3AnhdV2dT(self, T: float, V: float) -> float:
        """
        Third order derivative of the intrinsic anharmonicity correction of the Debye temperature,
        with respect to T, V, and V.

        :param float T: Temperature
        :param float V: Volume
        :return: dAnhdV_T_val/V*(m0/T-1/T+3/2*an_val*m0-an_val/2+an_val^2*m0*T/4)
        :rtype: float
        """
        self.d3AnhdV2dT_val = self.dAnhdV_T_val / V * (
                    self.m0 / T - 1 / T + 3 / 2 * self.an_val * self.m0 - self.an_val / 2 + self.an_val ** 2 * self.m0 * T / 4)
        return self.d3AnhdV2dT_val



from PySide6.QtWidgets import QDialog
from debyetools.tpropsgui.ui_warning import Ui_Dialog as Ui_Warning

class dialogWarning(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_Warning()
        self.ui.setupUi(self)



import numpy as np
np.seterr(divide='ignore')

kB   = 0.138064852e-22
NAv  = 0.6022140857e24

class Defects:
    """
    Implementation of the defects contribution due to monovancies to the free energy.

    :param float Evac00: Fomration energy of vacancies.
    :param float Svac00: Fomration entropy of vacancies.
    :param float Tm: Melting temperature.
    :param float a: Volume ratio of a mono vacancie relative to the equilibrium volume.
    :param float P2: Bulk modulus.
    :param float V0: Equilibrium volume.
    """
    def __init__(self, Evac00: float, Svac00: float, Tm: float, a: float, P2: float, V0: float):
        self.pdef = Evac00,Svac00,Tm,a,P2,V0
        self.Evac00 = Evac00
        self.Svac00 = Svac00
        self.Svac0 = Svac00*kB
        self.Evac0 = Evac00*kB*Tm
        self.Tm=Tm
        self.a = a
        self.P2 = P2
        self.V0=V0

    def Svac(self, V: float) -> float:
        """
        Entropy of formation of vacancies.

        :param float V: Volume.
        :return: Svac0
        :rtype: float
        """

        return self.Svac0
    def dSvacdV_T(self, V: float) -> float:
        """
        Volume-derivative of the entropy of formation of vacancies.

        :param float V: Volume.
        :return: 0
        """
        return 0
    def d2SvacdV2_T(self, V: float) -> float:
        """
        Volume-derivative of the entropy of formation of vacancies.

        :param float V: Volume.
        :return: 0
        """
        return 0
    def d3SvacdV3_T(self, V: float) -> float:
        """
        Volume-derivative of the entropy of formation of vacancies.

        :param float V: Volume.
        :return: 0
        """
        return 0
    def d4SvacdV4_T(self, V: float) -> float:
        """
        Volume-derivative of the entropy of formation of vacancies.

        :param float V: Volume.
        :return: 0
        """
        return 0
    def Evac(self, V: float) -> float:
        """
        Enthalpy of formation of vacancies.

        :param float V: Volume.
        :return: Ef(V)
        :rtype: float
        """
        return self.Evac0 - self.V0*self.a*(V - self.V0)*self.P2/(NAv*V)
    def dEvacdV_T(self, V: float) -> float:
        """
        Volume-derivative of the enthalpy of formation of vacancies.

        :param float V: Volume.
        :return: Volume-derivative of the enthalpy of formation of vacancies.
        :rtype: float
        """
        return -self.V0*self.a*self.P2/(NAv*V)+self.V0*self.a*(V-self.V0)*self.P2/(NAv*V**2)
    def d2EvacdV2_T(self, V: float) -> float:
        """
        Volume-derivative of the enthalpy of formation of vacancies.

        :param float V: Volume.
        :return: Volume-derivative of the enthalpy of formation of vacancies.
        :rtype: float
        """
        return 2*self.V0*self.a*self.P2/(NAv*V**2)-2*self.V0*self.a*(V-self.V0)*self.P2/(NAv*V**3)
    def d3EvacdV3_T(self, V: float) -> float:
        """
        Volume-derivative of the enthalpy of formation of vacancies.

        :param float V: Volume.
        :return: Volume-derivative of the enthalpy of formation of vacancies.
        :rtype: float
        """
        return -6*self.V0*self.a*self.P2/(NAv*V**3)+6*self.V0*self.a*(V-self.V0)*self.P2/(NAv*V**4)
    def d4EvacdV4_T(self, V: float) -> float:
        """
        Volume-derivative of the enthalpy of formation of vacancies.

        :param float V: Volume.
        :return: Volume-derivative of the enthalpy of formation of vacancies.
        :rtype: float
        """
        return 24*self.V0*self.a*self.P2/(NAv*V**4)-24*self.V0*self.a*(V-self.V0)*self.P2/(NAv*V**5)

    def E(self, T: float, V: float) -> float:
        """
        Defects energy.

        :param float T: Temperature.
        :param float V: Volume
        :return: E_def
        :rtype: float
        """
        return self.Evac(V)*NAv*np.exp(self.Svac(V)/kB - self.Evac(V)/(kB*T))
    def S(self, T: float, V: float) -> float:
        """
        Defects entropy.

        :param float T: Temperature.
        :param float V: Volume
        :return: S_def
        :rtype: float
        """
        return (T*kB+self.Evac(V))*NAv*np.exp(self.Svac(V)/kB - self.Evac(V)/(kB*T))/T
    def F(self, T: float, V: float) -> float:
        """
        Implementation of the defects contribution to the free energy.

        :param float T: Temperature.
        :param float V:  Volume.
        :return: energy
        :rtype: float
        """
        return -NAv*T*kB*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))
    def dFdV_T(self, T: float, V: float) -> float:
        """
        Derivative of defects contribution to the free energy.

        :param float T: Temperature.
        :param float V:  Volume.
        :return: Derivative of F_def
        :rtype: float
        """
        return -NAv*(self.dSvacdV_T(V)*T-self.dEvacdV_T(V))*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))

    def dFdT_V(self, T: float, V: float) -> float:
        """
        Derivative of defects contribution to the free energy.

        :param float T: Temperature.
        :param float V:  Volume.
        :return: Derivative of F_def
        :rtype: float
        """
        return -NAv*kB*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))-NAv*T*kB*(self.Svac(V)/(T*kB)-(self.Svac(V)*T-self.Evac(V))/(T**2*kB))*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))
    def d2FdT2_V(self, T: float, V: float) -> float:
        """
        Derivative of defects contribution to the free energy.

        :param float T: Temperature.
        :param float V:  Volume.
        :return: Derivative of F_def
        :rtype: float
        """
        return -NAv*self.Evac(V)**2*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))/(T**3*kB)
    def d2FdV2_T(self, T: float, V: float) -> float:
        """
        Derivative of defects contribution to the free energy.

        :param float T: Temperature.
        :param float V:  Volume.
        :return: Derivative of F_def
        :rtype: float
        """
        return -(-(self.d2EvacdV2_T(V))*T*kB+(self.d2SvacdV2_T(V))*T**2*kB+((self.dSvacdV_T(V))*T-(self.dEvacdV_T(V)))**2)*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))*NAv/(T*kB)
    def d3FdV3_T(self, T: float, V: float) -> float:
        """
        Derivative of defects contribution to the free energy.

        :param float T: Temperature.
        :param float V:  Volume.
        :return: Derivative of F_def
        :rtype: float
        """
        return -NAv*(self.d3SvacdV3_T(V)*T-self.d3EvacdV3_T(V))*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))-3*NAv*(self.d2SvacdV2_T(V)*T-self.d2EvacdV2_T(V))*(self.dSvacdV_T(V)*T-self.dEvacdV_T(V))*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))/(T*kB)-NAv*(self.dSvacdV_T(V)*T-self.dEvacdV_T(V))**3*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))/(T**2*kB**2)
    def d4FdV4_T(self, T: float, V: float) -> float:
        """
        Derivative of defects contribution to the free energy.

        :param float T: Temperature.
        :param float V:  Volume.
        :return: Derivative of F_def
        :rtype: float
        """
        return -NAv*(self.d4SvacdV4_T(V)*T-self.d4EvacdV4_T(V))*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))-4*NAv*(self.d3SvacdV3_T(V)*T-self.d3EvacdV3_T(V))*(self.dSvacdV_T(V)*T-self.dEvacdV_T(V))*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))/(T*kB)-3*NAv*(self.d2SvacdV2_T(V)*T-self.d2EvacdV2_T(V))**2*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))/(T*kB)

    def d2FdVdT(self, T: float, V: float) -> float:
        """
        Derivative of defects contribution to the free energy.

        :param float T: Temperature.
        :param float V:  Volume.
        :return: Derivative of F_def
        :rtype: float
        """
        return -np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))*(T*(T*kB+self.Evac(V))*(self.dSvacdV_T(V))-(self.dEvacdV_T(V))*self.Evac(V))*NAv/(T**2*kB)
    def d3FdV2dT(self, T: float, V: float) -> float:
        """
        Derivative of defects contribution to the free energy.

        :param float T: Temperature.
        :param float V:  Volume.
        :return: Derivative of F_def
        :rtype: float
        """
        return -NAv*(self.d2SvacdV2_T(V))*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))-NAv*((self.d2SvacdV2_T(V))*T-(self.d2EvacdV2_T(V)))*(self.Svac(V)/(T*kB)-(self.Svac(V)*T-self.Evac(V))/(T**2*kB))*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))-2*NAv*((self.dSvacdV_T(V))*T-(self.dEvacdV_T(V)))*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))*(self.dSvacdV_T(V))/(T*kB)+NAv*((self.dSvacdV_T(V))*T-(self.dEvacdV_T(V)))**2*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))/(T**2*kB)-NAv*((self.dSvacdV_T(V))*T-(self.dEvacdV_T(V)))**2*(self.Svac(V)/(T*kB)-(self.Svac(V)*T-self.Evac(V))/(T**2*kB))*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))/(T*kB)
    def d3FdVdT2(self, T: float, V: float) -> float:
        """
        Derivative of defects contribution to the free energy.

        :param float T: Temperature.
        :param float V:  Volume.
        :return: Derivative of F_def
        :rtype: float
        """
        return -2*NAv*self.dSvacdV_T(V)*(self.Svac(V)/(T*kB)-(self.Svac(V)*T-self.Evac(V))/(T**2*kB))*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))-NAv*(self.dSvacdV_T(V)*T-self.dEvacdV_T(V))*(-2*self.Svac(V)/(T**2*kB)+(2*(self.Svac(V)*T-self.Evac(V)))/(T**3*kB))*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))-NAv*(self.dSvacdV_T(V)*T-self.dEvacdV_T(V))*(self.Svac(V)/(T*kB)-(self.Svac(V)*T-self.Evac(V))/(T**2*kB))**2*np.exp((self.Svac(V)*T-self.Evac(V))/(T*kB))
